     //////////////////////////////////////////////////////////////////////
     //
     // Copyright (C) 1998 Analytic Services, Inc. (ANSER)
     //
     // Permission is hereby granted, free of charge, to any person
     // obtaining a copy of this software and associated documentation
     // files (the "Software"), to deal in the Software without
     // restriction, including without limitation the rights to use, copy,
     // modify, merge, publish, distribute, sublicense, and/or sell copies
     // of the Software, and to permit persons to whom the Software is
     // furnished to do so, subject to the following conditions:
     //
     // The above copyright notice and this permission notice shall be
     // included in all copies or substantial portions of the Software.
     //
     // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
     // EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
     // MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
     // NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
     // HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
     // WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
     // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
     // DEALINGS IN THE SOFTWARE.
     //
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mit.app.ajax;

     import java.io.*;
     import java.net.*;
     import java.util.*;

     import javax.servlet.*;
     import javax.servlet.http.*;

     import com.orbs.open1.mit.io.FileLib;

     /*********************************************************************
     * 
     * A superclass for AJAX servlets that provides a common HTML header
     * and footer and the option of loading user authentication data.
     * 
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft/">David W. Croft</A>
     * @version
     *   1998-10-29
     *********************************************************************/

     public abstract class  AJAXServlet extends HttpServlet
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     protected static final String  AJAX_HEADER_ALIAS
       = "/java/app/ajax/header.html";
     protected static final String  AJAX_FOOTER_ALIAS
       = "/java/app/ajax/footer.html";

     /** Singleton */
     protected static UserManager  userManager;
                                                        
     protected String  header;
     protected String  footer;

     //////////////////////////////////////////////////////////////////////
     // Static methods
     //////////////////////////////////////////////////////////////////////

     public static String  initHeader (
       ServletContext  servletContext,
       String          header_alias,
       String          title )
       throws IOException
     //////////////////////////////////////////////////////////////////////
     {
       StringBuffer  header = new StringBuffer ( );

       header.append ( "<HTML>\n<HEAD>\n<TITLE>\n" + title
         + "\n</TITLE>\n</HEAD>\n" );

       String  ajax_header_real
         = servletContext.getRealPath ( AJAX_HEADER_ALIAS );
       try
       {
         header.append ( FileLib.readTextFile ( ajax_header_real ) );
       }
       catch ( FileNotFoundException  e )
       {
         header.append ( "<!Missing " + AJAX_HEADER_ALIAS + ">\n"
           + "<BODY>\n" );
       }

       if ( header_alias != null )
       {
         String  header_real
           = servletContext.getRealPath ( header_alias );
         try
         {
           header.append ( FileLib.readTextFile ( header_real ) );
         }
         catch ( FileNotFoundException  e )
         {
           header.append ( "<!Missing " + header_alias + ">\n" );
         }
       }

       return header.toString ( );
     }

     public static String  initFooter (
       ServletContext  servletContext, String  footer_alias )
       throws IOException
     //////////////////////////////////////////////////////////////////////
     {
       StringBuffer  footer = new StringBuffer ( );

       if ( footer_alias != null )
       {
         String  footer_real
           = servletContext.getRealPath ( footer_alias );
         try
         {
           footer.append ( FileLib.readTextFile ( footer_real ) );
         }
         catch ( FileNotFoundException  e )
         {
           footer.append ( "<!Missing " + footer_alias + ">\n" );
         }
       }

       String  ajax_footer_real
         = servletContext.getRealPath ( AJAX_FOOTER_ALIAS );
       try
       {
         footer.append ( FileLib.readTextFile ( ajax_footer_real ) );
       }
       catch ( FileNotFoundException  e )
       {
         footer.append ( "<!Missing " + AJAX_FOOTER_ALIAS + ">\n" );
       }

       footer.append ( "</BODY>\n</HTML>" );

       return footer.toString ( );
     }

     public static UserData  getUserData (
       HttpServletRequest   req,
       PrintStream          out )
       throws IOException
     //////////////////////////////////////////////////////////////////////
     {
       String  username = req.getRemoteUser ( );
       if ( ( username == null ) || ( username.length ( ) == 0 ) )
       {
         out.println ( "<P><BLINK><B>Invalid username.</B></BLINK><P>" );
         out.println ( "This may be a result of not having the web server "
           + "protections set up that require a user name and password." );
         out.println ( "<P>Go to security, resources, defaultRealm." );
         out.println ( "<P>Select Add, Basic, agentUserACL, Servlet." );
         out.println ( "</BODY></HTML>" );
         return null;
       }
       UserData  userData = getUserManager ( ).getUserData ( username );
       if ( userData == null )
       {
         out.println ( "<P><BLINK><B>Null user data.</B></BLINK><BR>" );
         out.println ( "Please re-register.<P>" );
         RegistrationServlet.page_register ( out, username, null, null );
         return null;
       }
       return userData;
     }

     public static UserManager  getUserManager ( )
     //////////////////////////////////////////////////////////////////////
     {
       return userManager;
     }

     private static synchronized void  createUserManager (
       ServletContext  servletContext )
     //////////////////////////////////////////////////////////////////////
     {
       if ( userManager != null ) return;
       userManager = new UserManager ( servletContext.getRealPath (
         AJAXConstants.USER_MANAGER_USER_DATA_FILENAME_ALIAS ) );
     }

     //////////////////////////////////////////////////////////////////////
     // Abstract methods
     //////////////////////////////////////////////////////////////////////

     /*********************************************************************
     * Returns the title to be used with the HTML header tag TITLE.
     *********************************************************************/
     protected abstract String  getTitle ( );

     /*********************************************************************
     * Returns the relative path and filename for the servlet-specific
     * header file.
     * May return null.
     * Example:  "/java/app/myservlet/header.html".
     *********************************************************************/
     protected abstract String  getHeaderAlias ( );

     /*********************************************************************
     * Returns the relative path and filename for the servlet-specific
     * footer file.
     * May return null.
     * Example:  "/java/app/myservlet/footer.html".
     *********************************************************************/
     protected abstract String  getFooterAlias ( );

     /*********************************************************************
     * Returns true if the servlet requires user authentication.
     *********************************************************************/
     protected abstract boolean  requiresUserData ( );

     /*********************************************************************
     * Called by doPost() to provide the servlet-specific services.
     * Note that the UserData passed as in as an argument will be null
     * if UserData is not required or if an error occurred while trying
     * to authenticate the user.
     *********************************************************************/
     protected abstract void  processRequest (
       HttpServletRequest   req,
       HttpServletResponse  res,
       UserData             userData )
       throws IOException, ServletException;

     //////////////////////////////////////////////////////////////////////
     // Instance methods
     //////////////////////////////////////////////////////////////////////

     /*********************************************************************
     * Calls the superclass method and then assigns instance variables
     * header and footer by calling getTitle(), initHeader(),
     * and initFooter().  Exceptions are caught and routed to the
     * standard error output.
     *********************************************************************/
     public void  init ( ServletConfig  config ) throws ServletException
     //////////////////////////////////////////////////////////////////////
     {
       try
       {
         super.init ( config );

         ServletContext  sc = getServletContext ( );

         header = initHeader ( sc, getHeaderAlias ( ), getTitle ( ) );
         footer = initFooter ( sc, getFooterAlias ( ) );

         if ( userManager == null ) createUserManager ( sc );
       }
       catch ( Exception  ex ) { ex.printStackTrace ( ); }
     }

     /*********************************************************************
     * Reroutes to doPost().
     *********************************************************************/
     public void  doGet (
       HttpServletRequest  req, HttpServletResponse  res )
       throws ServletException, IOException
     //////////////////////////////////////////////////////////////////////
     {
       doPost ( req, res );
     }

     /*********************************************************************
     * Calls sendHtmlHeader(), processRequest(), and then sendHtmlFooter().
     * If a call to requiresUserData() returns true, will call
     * getUserData() and pass the UserData object to processRequest().
     *********************************************************************/
     public void  doPost (
       HttpServletRequest   req,
       HttpServletResponse  res )
       throws IOException, ServletException
     //////////////////////////////////////////////////////////////////////
     {
       res.setContentType ( "text/html" );
       // Java 1.2:  PrintWriter  out = res.getWriter ( );
       PrintStream  out = new PrintStream ( res.getOutputStream ( ) );
       try
       {
         sendHtmlHeader ( out );
         out.flush ( );

         UserData  userData = null;
         if ( requiresUserData ( ) ) userData = getUserData ( req, out );

         processRequest ( req, res, userData );
       }
       catch ( Throwable  ex )
       {
         ex.printStackTrace ( );
         System.out.println ( ex.getMessage ( ) );
         out.println ( "<P><PRE>" );
         // Will be deprecated until 1.2 is available to support
         //   res.getWriter().
         // "new PrintWriter ( res.getOutputStream ( ) )" doesn't work.
         ex.printStackTrace ( out );
         out.println ( "</PRE><P>" );
         out.flush ( );
       }
       finally
       {
         try
         {
           sendHtmlTrailer ( out );
           out.flush ( );
         }
         catch ( Exception  ex ) { }
       }
     }

     /*********************************************************************
     * out.println ( header );
     * <P>
     * May be overridden for additional header formatting.
     *********************************************************************/
     protected void  sendHtmlHeader (
       PrintStream  out ) throws IOException
     //////////////////////////////////////////////////////////////////////
     {
       out.println ( header );
     }

     /*********************************************************************
     * out.println ( footer );
     * <P>
     * May be overridden for additional footer formatting.
     *********************************************************************/
     protected void  sendHtmlTrailer (
       PrintStream  out ) throws IOException
     //////////////////////////////////////////////////////////////////////
     {
       out.println ( footer );
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
