     //////////////////////////////////////////////////////////////////////
     //
     // Copyright (C) 1998 Analytic Services, Inc. (ANSER)
     //
     // Permission is hereby granted, free of charge, to any person
     // obtaining a copy of this software and associated documentation
     // files (the "Software"), to deal in the Software without
     // restriction, including without limitation the rights to use, copy,
     // modify, merge, publish, distribute, sublicense, and/or sell copies
     // of the Software, and to permit persons to whom the Software is
     // furnished to do so, subject to the following conditions:
     //
     // The above copyright notice and this permission notice shall be
     // included in all copies or substantial portions of the Software.
     //
     // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
     // EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
     // MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
     // NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
     // HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
     // WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
     // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
     // DEALINGS IN THE SOFTWARE.
     //
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mit.app.calendarcourtier;

     import java.io.IOException;
     import java.util.Date;

     import com.orbs.open1.mit.net.mail.SMTPMail;
// Get rid of SMTPTest.
     import com.orbs.open1.mit.net.mail.SMTPTest;
     import com.orbs.open1.mit.servlet.Mission;

     /*********************************************************************
     * <P>
     * @author                                                        
     *   <A HREF="http://www.alumni.caltech.edu/~croft">David W. Croft</A>
     * @version
     *   1998-04-21
     *********************************************************************/

     public class  CalendarCourtierMission extends Mission {
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     private static final String  SUBJECT = "Calendar Courtier";

     static final long serialVersionUID = -306841967598531180L;

     public static final int  TYPE_ONCE     = 0;
     public static final int  TYPE_YEARLY   = 1;
     public static final int  TYPE_MONTHLY  = 2;
     public static final int  TYPE_BIWEEKLY = 3;
     public static final int  TYPE_WEEKLY   = 4;
     public static final int  TYPE_DAILY    = 5;

     private String  username;
     private String  usermail;
     private String  message;
     private int     type;

     //////////////////////////////////////////////////////////////////////
     // Constructor Methods
     //////////////////////////////////////////////////////////////////////
        
     public  CalendarCourtierMission (
       String  username,
       String  usermail,
       String  message,
       int     type,
       Date    start ) {
     //////////////////////////////////////////////////////////////////////
       this.username = username;
       this.usermail = usermail;
       this.message  = message;
       this.type     = type;
       setNextRun ( start );
     }

     public String  getUsername ( ) { return username; }
     public String  getUsermail ( ) { return usermail; }
     public String  getMessage  ( ) { return message ; }

     public String  getSchedule ( ) {
     //////////////////////////////////////////////////////////////////////
       switch ( type ) {
         case TYPE_ONCE    : return "Once";
         case TYPE_YEARLY  : return "Yearly";
         case TYPE_MONTHLY : return "Monthly";
         case TYPE_BIWEEKLY: return "Biweekly";
         case TYPE_WEEKLY  : return "Weekly";
         case TYPE_DAILY   : return "Daily";
         default           : return "Unknown";
       }
     }

     public synchronized void  run ( ) {
     //////////////////////////////////////////////////////////////////////
       if ( !isRunDue ( ) ) return;
       setLastRun ( new Date ( ) );
       try {
         SMTPMail.send (
           SMTPTest.DEFAULT_MAIL_SERVER_HOST,
           SMTPTest.DEFAULT_ADDRESS_FROM,
           usermail, SUBJECT, message );
       } catch ( Exception  e ) { setNextRun ( null ); }
       Date  nextRun = getNextRun ( );
       while ( isRunDue ( ) ) {
         switch ( type ) {
           case TYPE_YEARLY  :
             nextRun.setYear  ( nextRun.getYear ( )  +  1 ); break;
           case TYPE_MONTHLY :
             nextRun.setMonth ( nextRun.getMonth ( ) +  1 ); break;
           case TYPE_BIWEEKLY:
             nextRun.setDate  ( nextRun.getDate ( )  + 14 ); break;
           case TYPE_WEEKLY  :
             nextRun.setDate  ( nextRun.getDate ( )  +  7 ); break;
           case TYPE_DAILY   :
             nextRun.setDate  ( nextRun.getDate ( )  +  1 ); break;
           case TYPE_ONCE    :
           default           :
             nextRun = null;
             break;
         }
       }
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
