     //////////////////////////////////////////////////////////////////////
     // Copyright (c) 1999 David Wallace Croft (croft@alumni.caltech.edu)
     // Copyright (c) 1997 Analytic Services, Inc. (ANSER)
     //
     // Permission is hereby granted, free of charge, to any person
     // obtaining a copy of this software and associated documentation
     // files (the "Software"), to deal in the Software without
     // restriction, including without limitation the rights to use, copy,
     // modify, merge, publish, distribute, sublicense, and/or sell copies
     // of the Software, and to permit persons to whom the Software is
     // furnished to do so, subject to the following conditions:
     //
     // The above copyright notice and this permission notice shall be
     // included in all copies or substantial portions of the Software.
     //
     // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
     // EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
     // MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
     // NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
     // HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
     // WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
     // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
     // DEALINGS IN THE SOFTWARE.
     //
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mit.app.hitcounter;

     import java.awt.AWTException;
     import java.io.*;
     import java.net.*;
     import java.util.*;

     import javax.servlet.*;
     import javax.servlet.http.*;

     import com.orbs.open1.mit.io.SerializableLib;
     import com.orbs.open1.mit.media.image.codec.gif.GIFEncoder;

     /*********************************************************************
     * Hit counter servlet.
     *
     * <P>
     *
     * Generates an image of a number indicating how many times the
     * referrer has called this servlet.
     *
     * <P>
     *
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft/">David W. Croft</A>
     * @version
     *   1999-09-13
     *********************************************************************/

     public final class  HitCounterServlet extends HttpServlet
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     private static final String  SERVLET_INFO    = "HitCounterServlet";

     private static final String  HIT_COUNTER_DAT = "HitCounter.dat";

     private static final String  HIT_COUNTER_BAK = "HitCounter.bak";

     private static final long    SAVE_PERIOD = 1000 * 60 * 60; // 1 hour

     //////////////////////////////////////////////////////////////////////
     // Instance variables
     //////////////////////////////////////////////////////////////////////

     private Map                 hitCounterSiteDataMap;

     private HitCounterSiteData  hitCounterSiteData;

     private long                lastSaveTime;

     private boolean             dataIsDirty = false;

     private long                lastSaveSiteHits;

     //////////////////////////////////////////////////////////////////////
     // Public methods
     //////////////////////////////////////////////////////////////////////

     public String  getServletInfo ( )
     //////////////////////////////////////////////////////////////////////
     {
       return SERVLET_INFO;
     }

     public void  init ( ServletConfig  config ) throws ServletException
     //////////////////////////////////////////////////////////////////////
     {
       super.init ( config );

       hitCounterSiteDataMap = null;

       try
       {
         hitCounterSiteDataMap
           = ( Map ) SerializableLib.load ( HIT_COUNTER_DAT );
       }
       catch ( FileNotFoundException  fnfx )
       {
         // Assume it is the first time to run this program.
       }
       catch ( Exception  ex )
       {
         ex.printStackTrace ( );
       }

       if ( hitCounterSiteDataMap == null )
       {
         try
         {
           hitCounterSiteDataMap
             = ( Map ) SerializableLib.load ( HIT_COUNTER_BAK );
         }
         catch ( FileNotFoundException  fnfx )
         {
           // Assume it is the first time to run this program.
         }
         catch ( Exception  ex )
         {
           ex.printStackTrace ( );
         }
       }

       if ( hitCounterSiteDataMap == null )
       {
         hitCounterSiteDataMap = new HashMap ( );
       }

       hitCounterSiteData
         = new HitCounterSiteData ( hitCounterSiteDataMap );

       lastSaveTime = new Date ( ).getTime ( );

       lastSaveSiteHits = hitCounterSiteData.getHits ( );
     }

     public void  doGet (
       HttpServletRequest   req,
       HttpServletResponse  res )
       throws ServletException, IOException
     //////////////////////////////////////////////////////////////////////
     {
       String  referrer = req.getHeader ( "Referer" );

       String  details  = req.getParameter ( "d" );

       if ( ( details == null )
         || details.equals ( "" ) )
       {
         generateImage  ( res, referrer );
       }
       else
       {
         generateReport ( res, referrer );
       }
     }

     public void  doPost (
       HttpServletRequest   req,
       HttpServletResponse  res )
       throws ServletException, IOException
     //////////////////////////////////////////////////////////////////////
     {
       doGet ( req, res );
     }

     public void  destroy ( )
     //////////////////////////////////////////////////////////////////////
     {
       if ( dataIsDirty )
       {
         saveHitCounterData ( );
       }

       super.destroy ( );
     }

     //////////////////////////////////////////////////////////////////////
     // Private methods
     //////////////////////////////////////////////////////////////////////

     private void  generateImage (
       HttpServletResponse  res,
       String               referrer )
       throws IOException
     //////////////////////////////////////////////////////////////////////
     {
       dataIsDirty = true;

       long  hits = hitCounterSiteData.incrementHits ( referrer );

       String  s = Long.toString ( hits );

       int  l = s.length ( );

       short  w = 10;

       short  h = 14;

       int [ ]  values = new int [ l * w * h ];

       int [ ] [ ]  pic;

       for ( int  i = 0; i < l; i++ )
       {
         switch ( s.charAt ( i ) )
         {
           case '0': pic = ZERO ; break;
           case '1': pic = ONE  ; break;
           case '2': pic = TWO  ; break;
           case '3': pic = THREE; break;
           case '4': pic = FOUR ; break;
           case '5': pic = FIVE ; break;
           case '6': pic = SIX  ; break;
           case '7': pic = SEVEN; break;
           case '8': pic = EIGHT; break;
           case '9': pic = NINE ; break;
           default : pic = null ;
         }

         for ( int  x = 0; x < w; x++ )
         {
           for ( int  y = 0; y < h; y++ )
           {
             int  z = y * l * w + ( i * w ) + x;

             values [ z ] = pic [ y ] [ x ];
           }
         }
       }

       GIFEncoder  gifEncoder = null;

       try
       {
         gifEncoder = new GIFEncoder ( ( short ) ( w * l ), h, values );
       }
       catch ( AWTException  e ) { }

       if ( gifEncoder != null )
       {
         res.setContentType ( "image/gif" );

         res.setHeader ( "Cache-Control", "no-cache" );

         res.setHeader ( "Pragma", "no-cache" );

         ServletOutputStream  out = res.getOutputStream ( );

         gifEncoder.Write ( out );

         out.flush ( );

         out.close ( );
       }

       long  now = new Date ( ).getTime ( );

       if ( now >= SAVE_PERIOD + lastSaveTime )
       {
         synchronized ( this )
         {
           if ( dataIsDirty )
           {
             saveHitCounterData ( );
           }
         }
       }
     }

     private void  generateReport (
       HttpServletResponse  res,
       String               referrer )
       throws IOException
     //////////////////////////////////////////////////////////////////////
     {
       res.setContentType ( "text/html" );

       ServletOutputStream  out = res.getOutputStream ( );

       String  name = HitCounterPageData.normalizeURLName ( referrer );

       out.println ( "<HTML><HEAD><TITLE>" );

       out.println ( "Hit Counter Report:  " + name );

       out.println ( "</TITLE></HEAD><BODY><CENTER>" );

       out.println ( "<H1>Hit Counter Report</H1>" );

       out.println (
         "The source code for this servlet is available from \n" );

       out.println ( "<A HREF=\"http://www.orbs.com/software/open/\">"
         + "ORBS</A>." );

       out.println ( "<P><HR><P>" );

       generatePageReport ( out, name );

       out.println ( "<P><HR><P>" );

       generateSiteReport ( out );

       out.println ( "</CENTER></BODY></HTML>" );
     }

     private void  generatePageReport (
       ServletOutputStream  out,
       String               name )
       throws IOException
     //////////////////////////////////////////////////////////////////////
     {
       long  hits = hitCounterSiteData.getHits ( name );

       Date  firstHitDate = hitCounterSiteData.getFirstHitDate ( name );

       out.println (
         "<H2><A HREF=\"" + name + "\">" + name + "</A></H2><H3>" );

       out.println ( hits + " hit(s) since " + firstHitDate + "</H3>" );

       if ( firstHitDate != null )
       {
         double  days
           = new Date ( ).getTime ( ) - firstHitDate.getTime ( );

         days /= 1000 * 60 * 60 * 24;

         out.println ( ( ( double ) Math.round ( days * 100 ) ) / 100
           + " days have passed since counting started<BR>" );

         out.println ( "with an average of" );

         long  avg = Math.round ( hits / days );

         out.println ( "<H3>" + avg + " hits per day</H3>" );
       }

     }

     private void  generateSiteReport ( ServletOutputStream  out )
       throws IOException
     //////////////////////////////////////////////////////////////////////
     {
       List  hitCounterPageDataList
         = new LinkedList ( hitCounterSiteDataMap.values ( ) );

       Collections.sort ( hitCounterPageDataList );

       HitCounterPageData [ ]  hitCounterPageDataArray
         = ( HitCounterPageData [ ] ) hitCounterPageDataList.toArray (
         new HitCounterPageData [ ] { } );

       out.println ( "<TABLE BORDER=1 CELLPADDING=6>" );

       out.println ( "<TH>Hits/Day</TH>" );

       out.println ( "<TH>Hits</TH>" );

       out.println ( "<TH>URL</TH>" );

       long  now = new Date ( ).getTime ( );

       for ( int  i = hitCounterPageDataArray.length - 1; i > -1; i-- )
       {
         HitCounterPageData  hitCounterPageData
           = hitCounterPageDataArray [ i ];

         double  hitsPerDay = hitCounterPageData.getHitsPerDay ( now );

         long    hits       = hitCounterPageData.getHits ( );

         String  name       = hitCounterPageData.getName ( );

         out.println ( "<TR>" );

         out.println ( "<TD>" + hitsPerDay + "</TD>" );

         out.println ( "<TD>" + hits       + "</TD>" );

         out.println (
           "<TD><A HREF=\"" + name + "\">" + name + "</A></TD>" );

         out.println ( "</TR>" );
       }

       out.println ( "</TABLE>" );

     }

     private synchronized void  saveHitCounterData ( )
     //////////////////////////////////////////////////////////////////////
     {
       long  newLastSaveSiteHits = hitCounterSiteData.getHits ( );

       System.out.println (
         ( newLastSaveSiteHits - lastSaveSiteHits )
         + " site hits since "
         + new Date ( lastSaveTime )
         + ".  Saving HitCounterServlet data to "
         + HIT_COUNTER_DAT + "..." );

       lastSaveSiteHits = newLastSaveSiteHits;

       lastSaveTime = new Date ( ).getTime ( );

       try
       {
         SerializableLib.save (
           HIT_COUNTER_DAT, ( Serializable ) hitCounterSiteDataMap );
       }
       catch ( Exception  ex )
       {
         ex.printStackTrace ( );
       }

       try
       {
         SerializableLib.save (
           HIT_COUNTER_BAK, ( Serializable ) hitCounterSiteDataMap );
       }
       catch ( Exception  ex )
       {
         ex.printStackTrace ( );
       }

       dataIsDirty = false;
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     private static final int [ ] [ ]  ZERO = {
       { 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000 },
       { 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000 } };

     private static final int [ ] [ ]  ONE = { 
       { 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000, 0x000000 },
       { 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000, 0x000000 },
       { 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000, 0x000000 },
       { 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000 } };

     private static final int [ ] [ ]  TWO = { 
       { 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000 },
       { 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000 },
       { 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000 } };

     private static final int [ ] [ ]  THREE = { 
       { 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000 },
       { 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000 },
       { 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000 },
       { 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000 } };

     private static final int [ ] [ ]  FOUR = { 
       { 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000 },
       { 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000 },
       { 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000 } };

     private static final int [ ] [ ]  FIVE = { 
       { 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000 },
       { 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000 },
       { 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000 },
       { 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000 },
       { 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000 },
       { 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000 },
       { 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000 },
       { 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000 } };

     private static final int [ ] [ ]  SIX = { 
       { 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000 },
       { 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000 },
       { 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000 },
       { 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000 } };

     private static final int [ ] [ ]  SEVEN = { 
       { 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000 },
       { 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000 } };

     private static final int [ ] [ ]  EIGHT = { 
       { 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000 },
       { 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000 },
       { 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000 } };

     private static final int [ ] [ ]  NINE = { 
       { 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000 },
       { 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000 },
       { 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000 },
       { 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000 },
       { 0x000000, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0xFFFFFF, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000 },
       { 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000, 0x000000 } };

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
