     //////////////////////////////////////////////////////////////////////
     // Copyright (c) 1999 David Wallace Croft (croft@alumni.caltech.edu)
     //
     // Permission is hereby granted, free of charge, to any person
     // obtaining a copy of this software and associated documentation
     // files (the "Software"), to deal in the Software without
     // restriction, including without limitation the rights to use, copy,
     // modify, merge, publish, distribute, sublicense, and/or sell copies
     // of the Software, and to permit persons to whom the Software is
     // furnished to do so, subject to the following conditions:
     //
     // The above copyright notice and this permission notice shall be
     // included in all copies or substantial portions of the Software.
     //
     // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
     // EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
     // MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
     // NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
     // HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
     // WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
     // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
     // DEALINGS IN THE SOFTWARE.
     //
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mit.app.hitcounter;

     import java.io.*;
     import java.util.*;

     /*********************************************************************
     * Hit counter data for a web site.
     *
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft/">David W. Croft</A>
     * @version
     *   1999-09-12
     *********************************************************************/

     public final class  HitCounterSiteData
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     private Map  map;

     //////////////////////////////////////////////////////////////////////
     // Constructor method
     //////////////////////////////////////////////////////////////////////

     public  HitCounterSiteData ( Map  map )
     //////////////////////////////////////////////////////////////////////
     {
       if ( map == null )
       {
         throw new IllegalArgumentException ( "null map" );
       }

       this.map = map;
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public long  getHits ( )
     //////////////////////////////////////////////////////////////////////
     {
       long  siteHits = 0;

       synchronized ( map )
       {
         Iterator  i = map.values ( ).iterator ( );

         while ( i.hasNext ( ) )
         {
           HitCounterPageData  hitCounterPageData
             = ( HitCounterPageData ) i.next ( );

           siteHits += hitCounterPageData.getHits ( );
         }
       }

       return siteHits;
     }

     public long  getHits ( String  name )
     //////////////////////////////////////////////////////////////////////
     {
       HitCounterPageData  hitCounterPageData = get ( name );

       if ( hitCounterPageData == null )
       {
         return 0;
       }

       return hitCounterPageData.getHits ( );
     }

     public Date  getFirstHitDate ( String  name )
     //////////////////////////////////////////////////////////////////////
     {
       HitCounterPageData  hitCounterPageData = get ( name );

       if ( hitCounterPageData == null )
       {
         return null;
       }

       return hitCounterPageData.getFirstHitDate ( );
     }

     public long  incrementHits ( String  name )
     //////////////////////////////////////////////////////////////////////
     {
       return getOrCreate ( name ).incrementHits ( );
     }

     //////////////////////////////////////////////////////////////////////
     // Private methods
     //////////////////////////////////////////////////////////////////////

     private HitCounterPageData  get ( String  name )
     //////////////////////////////////////////////////////////////////////
     {
       return ( HitCounterPageData ) map.get (
         HitCounterPageData.normalizeURLName ( name ) );
     }

     private HitCounterPageData  getOrCreate ( String  name )
     //////////////////////////////////////////////////////////////////////
     {
       name = HitCounterPageData.normalizeURLName ( name );

       HitCounterPageData  hitCounterPageData = null;

       synchronized ( map )
       {
         hitCounterPageData = ( HitCounterPageData ) map.get ( name );

         if ( hitCounterPageData == null )
         {
           hitCounterPageData = new HitCounterPageData ( name );

           map.put ( name, hitCounterPageData );
         }
       }

       return hitCounterPageData;
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
