     //////////////////////////////////////////////////////////////////////
     //
     // Copyright (C) 1998 Analytic Services, Inc. (ANSER)
     //
     // Permission is hereby granted, free of charge, to any person
     // obtaining a copy of this software and associated documentation
     // files (the "Software"), to deal in the Software without
     // restriction, including without limitation the rights to use, copy,
     // modify, merge, publish, distribute, sublicense, and/or sell copies
     // of the Software, and to permit persons to whom the Software is
     // furnished to do so, subject to the following conditions:
     //
     // The above copyright notice and this permission notice shall be
     // included in all copies or substantial portions of the Software.
     //
     // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
     // EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
     // MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
     // NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
     // HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
     // WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
     // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
     // DEALINGS IN THE SOFTWARE.
     //
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mit.app.tidbits;

     import java.io.*;
     import java.net.*;
     import java.util.*;

     import javax.servlet.*;
     import javax.servlet.http.*;

     import com.orbs.open1.mit.app.ajax.*;
     import com.orbs.open1.mit.io.Serialize;
     import com.orbs.open1.mit.lang.ParseLib;
     import com.orbs.open1.mit.servlet.AgentServlet;
     import com.orbs.open1.mit.util.PairVector;
     import com.orbs.open1.mit.util.Sort;

     /*********************************************************************
     * <P>
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft">David W. Croft</A>
     * @version
     *   1998-05-05
     *********************************************************************/

     public class  TidBitsServlet extends AgentServlet
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     public static final String  TITLE   = "TidBits";
     public static final String  VERSION = "0.4.2";
     public static final String  INFO    = TITLE + " v" + VERSION
       + " Copyright 1998 ANSER";
     public static final String  MISSION_FILENAME = "TidBitsMission.data";
     public static final String  TIDBITS_FILENAME = "TidBitsRecord.data";
     public static final String  ACTION_URL = "/servlet/tidbitsservlet";

     public static final String  HEADER_ALIAS
       = "/java/app/tidbits/header.html";
     public static final String  FOOTER_ALIAS
       = "/java/app/tidbits/footer.html";

     public static final String [ ]  STARTERS = {
       "http://www.alumni.caltech.edu/~croft/",
       "http://nexos.anser.org:8080/",
       "http://www.alumni.caltech.edu/~croft/research/agent/",
       "http://www.alumni.caltech.edu/~croft/research/agent/definition/",
       "http://www.alumni.caltech.edu/~croft/research/java/",
       "http://www.alumni.caltech.edu/~croft/research/java/cache/",
       "http://www.anser.org/" };

     public static final int  HEIGHT_MIN =  5;
     public static final int  HEIGHT_MAX = 30;

     public static final String  PARAM_ADD    = "a";
     public static final String  PARAM_CONFIG = "c";
     public static final String  PARAM_FRAME  = "f";
     public static final String  PARAM_HEIGHT = "g";
     public static final String  PARAM_HELP   = "h";
     public static final String  PARAM_LIST   = "i";
     public static final String  PARAM_MAIL   = "m";
     public static final String  PARAM_URL    = "u";
     public static final String  PARAM_VOTE   = "v";

     private PairVector  pairVector;

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public void  init ( ServletConfig  config ) throws ServletException
     //////////////////////////////////////////////////////////////////////
     {
       super.init ( config );
       try
       {
         pairVector = ( PairVector ) Serialize.load ( TIDBITS_FILENAME );
         if ( ( pairVector == null ) || ( pairVector.size ( ) < 1 ) )
         {
           pairVector = new PairVector ( );

           for ( int  i = 0; i < STARTERS.length; i++ )
           {
             pairVector.put (
               STARTERS [ i ], new TidBitsRecord ( STARTERS [ i ] ) );
           }
         }
       }
       catch ( Exception  ex )
       {
         ex.printStackTrace ( );
       }
     }

     /*********************************************************************
     * Empty method to override inherited method.
     *********************************************************************/
     protected void  sendHtmlHeader (
       PrintStream  out ) throws IOException
     //////////////////////////////////////////////////////////////////////
     {
     }

     /*********************************************************************
     * Empty method to override inherited method.
     *********************************************************************/
     protected void  sendHtmlTrailer (
       PrintStream  out ) throws IOException
     //////////////////////////////////////////////////////////////////////
     {
     }

     //////////////////////////////////////////////////////////////////////
     // Abstract Implementation Methods
     //////////////////////////////////////////////////////////////////////

     public String   getTitle           ( ) { return INFO; }
     public String   getVersion         ( ) { return VERSION; }
     public String   getServletInfo     ( ) { return INFO; }
     public String   getHeaderAlias     ( ) { return HEADER_ALIAS; }
     public String   getFooterAlias     ( ) { return FOOTER_ALIAS; }
     public String   getMissionFilename ( ) { return MISSION_FILENAME; }

     public void  processRequest (
       HttpServletRequest   req,
       HttpServletResponse  res,
       UserData             userData,
       Vector               mission_Vector ) throws IOException
     //////////////////////////////////////////////////////////////////////
     {
       PrintStream  out = new PrintStream ( res.getOutputStream ( ) );

       String  param_url = req.getParameter ( PARAM_URL );
       if ( ( param_url == null ) || param_url.equals ( "" ) ) {
         page_frame ( out, userData, selectURL ( ) );
         return;
       }

       String  param_help = req.getParameter ( PARAM_HELP );
       if ( ( param_help != null ) && param_help.equals ( "1" ) ) {
         page_help ( out, userData, param_url );
         return;
       }

       String  param_frame = req.getParameter ( PARAM_FRAME );
       if ( ( param_frame != null ) && param_frame.equals ( "1" ) ) {
         page_frame ( out, userData, param_url );
         return;
       }

       String  param_mail = req.getParameter ( PARAM_MAIL );
       if ( ( param_mail != null ) && !param_mail.equals ( "" ) ) {
         page_mail ( out, userData, param_url, param_mail );
         return;
       }

       String  param_height = req.getParameter ( PARAM_HEIGHT );
       if ( ( param_height != null ) && !param_height.equals ( "" ) ) {
         page_height ( out, userData, param_url, param_height );
         return;
       }

       String  param_list = req.getParameter ( PARAM_LIST );
       if ( ( param_list != null ) && !param_list.equals ( "" ) ) {
         page_list ( out, userData, param_url );
         return;
       }

       String  param_config = req.getParameter ( PARAM_CONFIG );
       if ( ( param_config != null ) && !param_config.equals ( "" ) ) {
         page_configure ( out, userData, param_url );
         return;
       }

       String  param_vote = req.getParameter ( PARAM_VOTE );
       if ( ( param_vote != null ) && !param_vote.equals ( "" ) ) {
         page_vote ( out, userData, param_url, param_vote );
         return;
       }

       String  param_add = req.getParameter ( PARAM_ADD );
       if ( ( param_add != null ) && param_add.equals ( "1" ) ) {
         page_add ( out, userData, param_url );
         return;
       }

       page_controls ( out, userData, param_url );
     }
  
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public void  page_add ( 
       PrintStream  out,
       UserData             userData,
       String               url_String ) throws IOException {
     //////////////////////////////////////////////////////////////////////
       URL  url = null;
       try {
         url = new URL ( url_String );
         url_String = url.toExternalForm ( );
       } catch ( MalformedURLException  e ) { }
       out.println (
         "<HTML><HEAD><TITLE>" + TITLE + "</TITLE></HEAD><BODY>" );
       if ( url != null ) {
         if ( pairVector.containsKey ( url_String ) ) {
           out.println (
             "The URL \"" + url_String + "\" was already added." );
         } else {
           TidBitsRecord  tidbit = new TidBitsRecord ( url_String,
             userData.getName ( ), userData.getMail ( ) );
           tidbit.vote ( userData.getName ( ), true );
           if ( putSorted ( tidbit, true ) ) {
             out.println (
               "The URL \"" + url_String + "\" has been added." );
           } else {
             out.println (
               "Saving the URL \"" + url_String + "\" failed." );
           }
         }
       } else {
         out.println ( "The URL \"" + url_String + "\" is invalid." );
       }
       continue_trailer ( out, url_String );
     }

     public void  page_configure ( 
       PrintStream  out,
       UserData             userData,
       String               url_String ) throws IOException {
     //////////////////////////////////////////////////////////////////////
       TidBitsMission  mission = getMission ( userData );
       boolean  mail = mission.getMail ( );
       int  height = mission.getHeight ( );
       out.println (
           "<HTML><HEAD><TITLE>" + TITLE + "</TITLE></HEAD><BODY>\n"
         + "<TABLE><TR><TD>\n"
         + "Daily e-mail is " + ( mail ? "ON" : "OFF" ) + "\n"
         + "</TD><TD>\n"
         + "<FORM action=\"" + ACTION_URL + "\" METHOD=POST TARGET=\"_parent\">\n"
         + "<INPUT TYPE=HIDDEN NAME=\"" + PARAM_MAIL + "\" VALUE=" + ( mail ? 0 : 1 ) + ">\n"
         + "<INPUT TYPE=SUBMIT VALUE=TOGGLE>\n"
         + "<INPUT TYPE=HIDDEN NAME=\"" + PARAM_URL  + "\" VALUE=\"" + url_String + "\">\n"
         + "</FORM>\n"
         + "</TD></TR><TR><TD>\n"
         + "Controls Screen Height % (" + HEIGHT_MIN
         + " to " + HEIGHT_MAX + "):\n"
         + "</TD><TD>\n"
         + "<FORM action=\"" + ACTION_URL + "\" METHOD=POST TARGET=\"_parent\">\n"
         + "<INPUT TYPE=TEXT NAME=\"" + PARAM_HEIGHT + "\" SIZE=2"
         + " VALUE=" + height + ">\n"
         + "<INPUT TYPE=HIDDEN NAME=\"" + PARAM_URL  + "\" VALUE=\"" + url_String + "\">\n"
         + "</FORM>\n"
         + "</TD></TR></TABLE>" );
       continue_trailer ( out, url_String );
     }

     public static void  page_controls ( 
       PrintStream  out,
       UserData             userData,
       String               url_String ) throws IOException {
     //////////////////////////////////////////////////////////////////////
       out.println (
           "<HTML><HEAD><TITLE>" + TITLE + "</TITLE></HEAD><BODY>\n"
         + "<TABLE ALIGN=CENTER><TR><TD>\n"
         + "<FORM action=\"" + ACTION_URL + "\" METHOD=POST TARGET=\"_parent\">\n"
         + "<INPUT TYPE=SUBMIT VALUE=HELP>\n"
         + "<INPUT TYPE=HIDDEN NAME=\"" + PARAM_HELP + "\" VALUE=1>"
         + "<INPUT TYPE=HIDDEN NAME=\"" + PARAM_URL  + "\" VALUE=\"" + url_String + "\">\n"
         + "</FORM>\n"
         + "</TD><TD>\n"
         + "<FORM action=\"" + ACTION_URL + "\" METHOD=POST TARGET=\"_parent\">\n"
         + "<INPUT TYPE=SUBMIT VALUE=LIST>\n"
         + "<INPUT TYPE=HIDDEN NAME=\"" + PARAM_LIST + "\" VALUE=1>"
         + "<INPUT TYPE=HIDDEN NAME=\"" + PARAM_URL  + "\" VALUE=\"" + url_String + "\">\n"
         + "</FORM>\n"
         + "</TD><TD>\n"
         + "<FORM action=\"" + ACTION_URL + "\" METHOD=POST TARGET=\"_parent\">\n"
         + "<INPUT TYPE=SUBMIT VALUE=CONFIGURE>\n"
         + "<INPUT TYPE=HIDDEN NAME=\"" + PARAM_CONFIG + "\">"
         + "<INPUT TYPE=HIDDEN NAME=\"" + PARAM_URL  + "\" VALUE=\"" + url_String + "\">\n"
         + "</FORM>\n"
         + "</TD><TD>\n"
         + "<FORM action=\"" + ACTION_URL + "\" METHOD=POST TARGET=\"_parent\">\n"
//       + "<input type=SUBMIT VALUE=ADD>\n"
         + "<INPUT TYPE=HIDDEN NAME=\"" + PARAM_ADD + "\" VALUE=1>\n"
         + "<INPUT TYPE=TEXT NAME=\"" + PARAM_URL + "\" SIZE=40"
         + " VALUE=\"" + url_String + "\">\n"
         + "</FORM>\n"
         + "</TD><TD>\n"
         + "<FORM action=\"" + ACTION_URL + "\" METHOD=POST TARGET=\"_parent\">\n"
         + "<INPUT TYPE=SUBMIT VALUE=YEA>\n"
         + "<INPUT TYPE=HIDDEN NAME=\"" + PARAM_VOTE + "\" VALUE=1>\n"
         + "<INPUT TYPE=HIDDEN NAME=\"" + PARAM_URL  + "\" VALUE=\"" + url_String + "\">\n"
         + "</FORM>\n"
         + "</TD><TD>\n"
         + "<FORM action=\"" + ACTION_URL + "\" METHOD=POST TARGET=\"_parent\">\n"
         + "<INPUT TYPE=SUBMIT VALUE=???>\n"
//       + "<INPUT TYPE=HIDDEN NAME=\"" + PARAM_VOTE + "\" VALUE=0>\n"
//       + "<INPUT TYPE=HIDDEN NAME=\"" + PARAM_URL  + "\" VALUE=\"" + url_String + "\">\n"
         + "</FORM>\n"
         + "</TD><TD>\n"
         + "<FORM action=\"" + ACTION_URL + "\" METHOD=POST TARGET=\"_parent\">\n"
         + "<INPUT TYPE=SUBMIT VALUE=NAY>\n"
         + "<INPUT TYPE=HIDDEN NAME=\"" + PARAM_VOTE + "\" VALUE=-1>\n"
         + "<INPUT TYPE=HIDDEN NAME=\"" + PARAM_URL  + "\" VALUE=\"" + url_String + "\">\n"
         + "</FORM>\n"
         + "</TD></TR></TABLE>\n"
         + "</BODY></HTML>" );
     }

     public void  page_frame ( 
       PrintStream  out,
       UserData     userData,
       String       url_String ) throws IOException
     //////////////////////////////////////////////////////////////////////
     {
       int  height = getMission ( userData ).getHeight ( );
       out.println (
           "<HTML><HEAD><TITLE>" + TITLE + "</TITLE></HEAD>\n"
         + "<FRAMESET ROWS=\"" + height + "%,*\">\n"
         + "<FRAME"
         + " SRC=\"" + ACTION_URL
         + "?" + PARAM_URL + "=" + url_String + "\">\n"
         + "<FRAME SRC=\"" + url_String + "\">\n"
         + "</FRAMESET>\n"
         + "<NOFRAME><HEAD><TITLE>" + TITLE + "</TITLE></HEAD>\n"
         + "<BODY>Requires frame-capable browser.</BODY></HTML>" );
     }

     public void  page_height ( 
       PrintStream  out,
       UserData             userData,
       String               url_String,
       String               height_String ) throws IOException {
     //////////////////////////////////////////////////////////////////////
       TidBitsMission  mission = getMission ( userData );
       int  height = ParseLib.parseInt (
         height_String, mission.getHeight ( ) );
       if ( height < HEIGHT_MIN ) height = HEIGHT_MIN;
       if ( height > HEIGHT_MAX ) height = HEIGHT_MAX;
       mission.setHeight ( height );
       save ( );
       page_configure ( out, userData, url_String );
       out.println ( "The height has been set to " + height );
     }

     public void  page_help ( 
       PrintStream  out,
       UserData             userData,
       String               url_String ) throws IOException {
     //////////////////////////////////////////////////////////////////////
       out.println (
           "<HTML><HEAD><TITLE>" + TITLE + "</TITLE></HEAD><BODY>\n"
         + "<H1>" + TITLE + "</H1>Version " + VERSION + "<BR>\n"
         + "<UL>\n"
         + "<LI><B>YEA</B>:  Increases the probability that the page\n"
         + "    will be loaded in the future.\n"
         + "<LI><B>NAY</B>:  Decreases the probability that the page\n"
         + "    will be loaded in the future then loads a random page.\n"
         + "<LI><B>???</B>:  Loads a random page.\n"
         + "<LI><B>LIST</B>:  Lists the available URLs with votes.\n"
         + "<LI><B>CONFIGURE</B>:  Configure user options such as\n"
         + "    control area height and sending daily e-mail.\n"
         + "<LI><B>Adding Pages</B>:  Enter a URL into the text field.\n"
         + "</UL>\n"
         + "<P><HR><P>\n"
         + "<A HREF=\"mailto:croft@alumni.caltech.edu\">David Croft</A>\n"
         );
       continue_trailer ( out, url_String );
     }

     public void  page_list ( 
       PrintStream  out,
       UserData             userData,
       String               url_String ) throws IOException {
     //////////////////////////////////////////////////////////////////////
       out.println (
           "<HTML><HEAD><TITLE>" + TITLE + "</TITLE></HEAD><BODY>\n"
         + "<CENTER>\n"
         + userData.getName ( ) + "<BR>\n"
         + userData.getMail ( ) + "<P>\n"
         + "<HR>" );
       Enumeration  e = pairVector.values ( );
       out.println ( "<TABLE ALIGN=CENTER BORDER>" );
       out.println ( "<THEAD><TH>Y<TH>N<TH>%<TH>URL<TH>USER<TBODY>\n" );
       while ( e.hasMoreElements ( ) ) {
         TidBitsRecord  tidbit = ( TidBitsRecord ) e.nextElement ( );
         double  percent = 100 * tidbit.getScore ( );
         out.println ( "<TR>" );
         out.println ( "<TD>" + tidbit.getYeas ( ) + "</TD>" );
         out.println ( "<TD>" + tidbit.getNays ( ) + "</TD>" );
         out.println ( "<TD>" + percent + "</TD>" );
         out.println ( "<TD><A HREF=\"" + ACTION_URL + "?"
           + PARAM_FRAME + "=1&"
           + PARAM_URL + "=" + tidbit.getName ( ) + "\">\n  "
           + tidbit.getName ( ) + "</A></TD>" );
         String  user = tidbit.getUser ( );
         out.println ( "<TD><A HREF=\"mailto:" + tidbit.getMail ( )
           + "\">" + ( user == null ? "" : user ) + "</A></TD>" );
         out.println ( "</TR>" );
       }
       out.println ( "</TABLE>" );
       continue_trailer ( out, url_String );
     }

     public void  page_mail ( 
       PrintStream  out,
       UserData             userData,
       String               url_String,
       String               mail_String ) throws IOException {
     //////////////////////////////////////////////////////////////////////
       TidBitsMission  mission = getMission ( userData );
       boolean  mail = mission.getMail ( );
       mail = ParseLib.parseBoolean ( mail_String, mail );
       mission.setMail ( mail );
       save ( );
       page_configure ( out, userData, url_String );
       out.println ( "The mail option has been set to " + mail );
     }

     public void  page_vote ( 
       PrintStream  out,
       UserData             userData,
       String               url_String,
       String               vote ) throws IOException {
     //////////////////////////////////////////////////////////////////////
       if ( !vote.equals ( "1" ) && !vote.equals ( "-1" ) ) {
         page_frame ( out, userData, selectURL ( ) );
         return;
       }
       out.println (
         "<HTML><HEAD><TITLE>" + TITLE + "</TITLE></HEAD><BODY>" );
       TidBitsRecord  tidbit
         = ( TidBitsRecord ) pairVector.get ( url_String );
       boolean  yea_nay = ParseLib.parseBoolean ( vote, false );
       tidbit.vote ( userData.getName ( ), yea_nay );
       out.println ( "<CENTER>" + userData.getName ( ) + " voted "
         + ( yea_nay ? "YEA" : "NAY" ) + " for<BR>\n"
         + "<A HREF=\"" + url_String + "\">" + url_String + "</A>" );
       if ( putSorted ( tidbit, true ) ) {
         out.println ( "<P>Vote was successfully saved." );
       } else {
         out.println ( "<P>Vote was NOT successfully saved." );
       }
       continue_trailer ( out, yea_nay ? url_String : selectURL ( ) );
     }

     //////////////////////////////////////////////////////////////////////
     // Other Methods
     //////////////////////////////////////////////////////////////////////

     public synchronized String  selectURL ( ) {
     //////////////////////////////////////////////////////////////////////
       int  size = pairVector.size ( );
       if ( size < 1 ) return "http://www.anser.org/";
       double  sum = 0.0;
       for ( int  i = 0; i < size; i++ ) {
         TidBitsRecord  tidbit
           = ( TidBitsRecord ) pairVector.valueAt ( i );
// This recalculates each time it is called.  Rewrite!
         sum += tidbit.getScore ( );
       }
       double  roll = sum * Math.random ( );
       double  cum = 0.0;
       for ( int  i = 0; i < size; i++ ) {
         TidBitsRecord  tidbit
           = ( TidBitsRecord ) pairVector.valueAt ( i );
         cum += tidbit.getScore ( );
         if ( roll < cum ) return tidbit.getName ( );
       }
       return "http://www.anser.org/";
     }

     public static void  continue_trailer (
       PrintStream  out,
       String               url_String ) throws IOException {
     //////////////////////////////////////////////////////////////////////
       out.println (
           "<FORM action=\"" + ACTION_URL
         + "\" METHOD=POST TARGET=\"_parent\">\n"
         + "<input type=SUBMIT VALUE=CONTINUE>\n"
         + "<INPUT TYPE=HIDDEN NAME=\"" + PARAM_URL
         + "\" VALUE=\"" + url_String + "\">\n"
         + "<INPUT TYPE=HIDDEN NAME=\"" + PARAM_FRAME + "\" VALUE=1>\n"
         + "</FORM>\n"
         + "</BODY></HTML>" );
     }

     public TidBitsMission  getMission ( UserData  userData ) {
     //////////////////////////////////////////////////////////////////////
       String  username = userData.getName ( );
       Vector  mission_Vector = ( Vector ) user_Hashtable.get ( username );
       if ( mission_Vector == null ) {
         mission_Vector = new Vector ( 1 );
         user_Hashtable.put ( username, mission_Vector );
       }
       TidBitsMission  mission = null;
       if ( mission_Vector.size ( ) < 1 ) {
         mission = new TidBitsMission (
           username, userData.getMail ( ), new Date ( ) );
         mission_Vector.addElement ( mission );
       }
       return ( TidBitsMission ) mission_Vector.firstElement ( );
     }

     public synchronized boolean  putSorted (
       TidBitsRecord  tidbit, boolean  save ) {
     //////////////////////////////////////////////////////////////////////
       pairVector.removeValue ( tidbit );
       double [ ]  scores = new double [ pairVector.size ( ) ];
       for ( int  i = 0; i < scores.length; i++ ) {
         scores [ i ] = ( ( TidBitsRecord )
           pairVector.valueAt ( i ) ).getScore ( );
       }
       pairVector.putAt ( tidbit.getName ( ), tidbit,
         Sort.rank ( tidbit.getScore ( ), scores ) );
       if ( !save ) return false;
       return Serialize.save ( TIDBITS_FILENAME, pairVector );
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
