     //////////////////////////////////////////////////////////////////////
     // Copyright (C) 1998 Analytic Services, Inc. (ANSER)
     //
     // Permission is hereby granted, free of charge, to any person
     // obtaining a copy of this software and associated documentation
     // files (the "Software"), to deal in the Software without
     // restriction, including without limitation the rights to use, copy,
     // modify, merge, publish, distribute, sublicense, and/or sell copies
     // of the Software, and to permit persons to whom the Software is
     // furnished to do so, subject to the following conditions:
     //
     // The above copyright notice and this permission notice shall be
     // included in all copies or substantial portions of the Software.
     //
     // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
     // EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
     // MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
     // NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
     // HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
     // WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
     // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
     // DEALINGS IN THE SOFTWARE.
     //
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mit.lang;

     import java.util.*;

     /*********************************************************************
     * 
     * A collection of static methods to manipulate java.lang.String.
     * 
     * @version
     *   1998-10-29
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft/">David W. Croft</A>
     *********************************************************************/

     public final class  StringLib
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     private  StringLib ( ) { }

     /*********************************************************************
     * Test method.
     *********************************************************************/
     public static final void  main ( String [ ]  args )
     //////////////////////////////////////////////////////////////////////
     {

       System.out.println ( "".indexOf ( "" ) );

       System.out.println ( "\"" + replace (
         "BILLBOB", "BILL", "BILLY" ) + "\"" );

       System.out.println ( "\"" + replace ( "ABC", "ABC", "" ) + "\"" );
       System.out.println ( "\"" + replace ( "ABC", "", "ABC" ) + "\"" );
       System.out.println ( "\"" + replace ( "", "ABC", "ABC" ) + "\"" );
       System.out.println ( "\"" + replace ( "", "", ""       ) + "\"" );
       System.out.println ( "\"" + replace ( "", "", "ABC"    ) + "\"" );

/*
       System.out.println ( toLowerCaseOutsideQuotes (
         "META HTTP-EQUIV=\"REFRESH\" CONTENT=\"1; URL=http://www.almost-heaven.com/lpwv/candidates/kerr/\""
         ) );

       System.out.println ( toBase32String ( new byte [ ] {    0 } ) );
       System.out.println ( toBase32String ( new byte [ ] {    1 } ) );
       System.out.println ( toBase32String ( new byte [ ] {   -1 } ) );
       System.out.println ( toBase32String ( new byte [ ] {   31 } ) );
       System.out.println ( toBase32String ( new byte [ ] {   32 } ) );
       System.out.println ( toBase32String ( new byte [ ] {   33 } ) );
       System.out.println ( toBase32String ( new byte [ ] {  127 } ) );
       System.out.println ( toBase32String ( new byte [ ] { -127 } ) );

       System.out.println ( '*' + "    ".trim ( ) + '*' );
       System.out.println ( '*' + ":".substring ( 0, 0 ) + '*' );
       System.out.println ( '*' + ":".substring ( 1 ) + '*' );
       System.out.println ( indexOfChar ( "f:oo: bar", ": r", 2 ) );
       System.out.println ( indexOfChar ( "f:oo: bar", "zippy", 2 ) );
       System.out.println ( "0*" + parseNameValue ( "A:B" ) [ 0 ] + "*" );
       System.out.println ( "1*" + parseNameValue ( "A:B" ) [ 1 ] + "*" );
       System.out.println ( "0*" + parseNameValue ( "A B" ) [ 0 ] + "*" );
       System.out.println ( "1*" + parseNameValue ( "A B" ) [ 1 ] + "*" );
       System.out.println ( "0*" + parseNameValue ( ": B" ) [ 0 ] + "*" );
       System.out.println ( "1*" + parseNameValue ( ": B" ) [ 1 ] + "*" );
       System.out.println ( "0*" + parseNameValue ( "A :" ) [ 0 ] + "*" );
       System.out.println ( "1*" + parseNameValue ( "A :" ) [ 1 ] + "*" );
       System.out.println ( "0*" + parseNameValue ( ":"   ) [ 0 ] + "*" );
       System.out.println ( "1*" + parseNameValue ( ":"   ) [ 1 ] + "*" );
*/
     }

     /*********************************************************************
     * Returns the left-most characters in <I>s</I> before the first
     * occurence of any character in <I>char_list</I>.
     *********************************************************************/
     public static String  breakAtChar ( String  s, String  char_list )
     //////////////////////////////////////////////////////////////////////
     {
       int  index = indexOfChar ( s, char_list, 0 );
       if ( index == -1 ) return new String ( s );
       return s.substring ( 0, index );
     }

     /*********************************************************************
     * Returns "new String(s)" or, if the argument s is null, null.
     * This is a more robust clone operation in that it can "clone" a
     * null String.
     *********************************************************************/
     public static String  clone ( String  s )
     //////////////////////////////////////////////////////////////////////
     {
       return ( s != null ) ? new String ( s ) : null;
     }

     /*********************************************************************
     * Converts a String of hexadecimal characters to an array of bytes.
     * @see #toHexString
     *********************************************************************/
     public static byte [ ]  hexStringToByteArray ( String  digest )
     //////////////////////////////////////////////////////////////////////
     {
       byte [ ]  b_array = new byte [ digest.length ( ) / 2 ];
       for ( int  i = 0; i < b_array.length; i++ )
       {
         int  ui = Integer.parseInt (
           digest.substring ( 2 * i, 2 * i + 2 ), 16 );
         if ( ui > 127 ) ui -= 256;
         b_array [ i ] = ( byte ) ui;
       }
       return b_array;
     }

     /*********************************************************************
     * As the java.lang.String.indexOf() method except that a mask
     * character in the "str" String can be used to indicate that any
     * character in the "s" String at the position is acceptable.  If
     * "maskEither" is true, then either or both strings can mask the
     * other.
     * <P>
     * Example:<BR>
     * indexOf ( "ABxDe", "bCXE", 0, true, 'x', true ) <BR>
     * returns 1
     *********************************************************************/
     public static int  indexOf (
       String  s, String  str, int  fromIndex, boolean  ignoreCase,
       char  maskChar, boolean  maskEither )
     //////////////////////////////////////////////////////////////////////
     {
       int  s_len = s.length ( );
       int  str_len = str.length ( );

       for ( int  i = fromIndex; i <= s_len - str_len; i++ )
       {
         if ( regionMatches (
           s, ignoreCase, i, str, 0, str_len, maskChar, maskEither ) )
           return i;
       }
       return -1;
     }

     /*********************************************************************
     * Returns indexOfChar ( s, char_list, 0 );
     *********************************************************************/
     public static int  indexOfChar ( String  s, String char_list )
     //////////////////////////////////////////////////////////////////////
     {
       return indexOfChar ( s, char_list, 0 );
     }

     /*********************************************************************
     * Returns the index of the first character found in <I>s</I> from
     * the list of characters in <I>char_list</I>.
     * Use java.lang.String.charAt ( int  index ) to determine which
     * character was returned.
     * Returns -1 if none of the characters in <I>char_list</I> were
     * found in <I>s</I>.
     *********************************************************************/
     public static int  indexOfChar (
       String  s, String char_list, int  fromIndex )
     //////////////////////////////////////////////////////////////////////
     {
       char [ ]  char_array = char_list.toCharArray ( );
       int  min = Integer.MAX_VALUE;
       boolean  found = false;
       for ( int  i = 0; i < char_array.length; i++ )
       {
         int  index = s.indexOf ( char_array [ i ], fromIndex );
         if ( ( index > -1 ) && ( index < min ) )
         {
           min = index;
           found = true;
         }
       }
       if ( found ) return min;
       return -1;
     }

     /*********************************************************************
     * Returns s.substring ( 0, length ).
     * @see #right
     *********************************************************************/
     public static String  left ( String  s, int  length )
     //////////////////////////////////////////////////////////////////////
     {
       return s.substring ( 0, length );
     }

     /*********************************************************************
     * Prepends character "c" to String "s" until the length of "s" is
     * "length".
     * @see #padRight
     *********************************************************************/
     public static String  padLeft ( String  s, char  c, int  length )
     //////////////////////////////////////////////////////////////////////
     {
       if ( s == null ) return null;
       if ( length <= s.length ( ) ) return new String ( s );

       while ( s.length ( ) < length ) s = c + s;
       return s;
     }

     /*********************************************************************
     * Appends character "c" to String "s" until the length of "s" is
     * "length".
     * @see #padLeft
     *********************************************************************/
     public static String  padRight ( String  s, char  c, int  length )
     //////////////////////////////////////////////////////////////////////
     {
       if ( s == null ) return null;
       if ( length <= s.length ( ) ) return new String ( s );

       while ( s.length ( ) < length ) s = s + c;
       return s;
     }

     /*********************************************************************
     * Parses a name and a value out of a name-value pair line delimited
     * by a colon (':').
     * The first array element, the name, is always returned in lower case
     * with the outer white space trimmed.
     * Anything after the colon is returned as the second array element,
     * the value, with the mixed-case preserved but with the outer white
     * space trimmed.
     *********************************************************************/
     public static String [ ]  parseNameValue ( String  line )
     //////////////////////////////////////////////////////////////////////
     {
       int  index = line.indexOf ( ':' );
       if ( index == -1 )
       {
         return new String [ ] { line.trim ( ).toLowerCase ( ), "" };
       }
       String  name = line.substring ( 0, index ).trim ( ).toLowerCase ( );
       String  value = line.substring ( index + 1 ).trim ( );
       return new String [ ] { name, value };
     }

     /*********************************************************************
     * As the java.lang.String.regionMatches() method except that a mask
     * character in the "other" String can be used to indicate that any
     * character in the "s" String at the position is acceptable.  If
     * "maskEither" is true, then either or both strings can mask the
     * other.
     * <P>
     * Example:<BR>
     * regionMatches ( "AbCx", true, 0, "aBXD", 0, 4, 'x', true ) <BR>
     * returns true
     *********************************************************************/
     public static boolean  regionMatches (
       String  s, boolean  ignoreCase, int  toffset, String  other,
       int  ooffset, int  len, char  maskChar, boolean  maskEither )
     //////////////////////////////////////////////////////////////////////
     {
       if ( ( toffset < 0 )
         || ( ooffset < 0 )
         || ( toffset + len > s.length ( )     )
         || ( ooffset + len > other.length ( ) ) ) return false;

       if ( ignoreCase ) maskChar = Character.toLowerCase ( maskChar );

       for ( int  k = 0; k < len; k++ )
       {
         char  s_char =     s.charAt ( toffset + k );
         char  o_char = other.charAt ( ooffset + k );

         if ( ignoreCase )
         {
           s_char = Character.toLowerCase ( s_char );
           o_char = Character.toLowerCase ( o_char );
         }

         if ( o_char == maskChar ) continue;
         if ( maskEither ) if ( s_char == maskChar ) continue;

         if ( s_char != o_char ) return false;
       }
       return true;
     }

     /*********************************************************************
     * Returns a String with the characters between
     * <I>fromIndex</I> and <I>toIndex</I> removed, inclusive.
     *********************************************************************/
     public static String  remove (
       String  s, int  fromIndex, int  toIndex )
     //////////////////////////////////////////////////////////////////////
     {
       return s.substring ( 0, fromIndex ) + s.substring ( toIndex + 1 );
     }

     /*********************************************************************
     * Replaces every instance of <I>oldSub</I> in <I>s</I> with
     * <I>newSub</I>.
     * Passes from left to right once.
     *********************************************************************/
     public static String  replace (
       String  s, String  oldSub, String  newSub )
     //////////////////////////////////////////////////////////////////////
     {
       StringBuffer  t = new StringBuffer ( );
       int  oldIndex = 0;
       int  index;

       int  oldSubLen = oldSub.length ( );
       if ( oldSubLen < 1 ) return s;
       int  sLen = s.length ( );

       while ( ( oldIndex < sLen )
         && ( ( index = s.indexOf ( oldSub, oldIndex ) ) > -1 ) )
       {
         if ( index > oldIndex )
         {
           t.append ( s.substring ( oldIndex, index ) );
         }
         t.append ( newSub );
         oldIndex = index + oldSubLen;
       }

       if ( oldIndex < sLen ) t.append ( s.substring ( oldIndex ) );

       return t.toString ( );
     }

     /*********************************************************************
     * Returns the rightmost <I>length</I> characters.
     * @see #left
     *********************************************************************/
     public static String  right ( String  s, int  length )
     //////////////////////////////////////////////////////////////////////
     {
       int  index = s.length ( ) - length;
       return s.substring ( index < 0 ? 0 : index );
     }

     /*********************************************************************
     * Splits a String at the first occurrence of the delimiter.
     * Useful for parsing name-value pairs.
     *
     * @return
     *    A String array composed of the left and right substrings at
     *    indices 0 and 1, respectively.  If the delimiter is not present,
     *    the second element will be null.  If the first occurrence of
     *    the delimiter is at the tail end of the String, the second
     *    element will be the empty String ("").
     *********************************************************************/
     public static String [ ]  splitAtDelimiter (
       String  pair, String  delimiter )
     //////////////////////////////////////////////////////////////////////
     {
       int  index = pair.indexOf ( delimiter );

       String  name, value;

       if ( index == -1 )
       {
         name  = pair;
         value = null;
       }
       else
       {
         name = pair.substring ( 0, index );

         if ( index + 1 == pair.length ( ) )
         {
           value = "";
         }
         else
         {
           value = pair.substring ( index + 1 );
         }
       }

       return new String [ ] { name, value };
     }



     /*********************************************************************
     * Trims whitespace then replaces all inner whitespace strings
     * with a single space character except within quotes.
     *********************************************************************/
     public static String  stripUnnecessaryWhitespace ( String  s )
     //////////////////////////////////////////////////////////////////////
     {
       if ( s == null ) return null;
       s = s.trim ( );
       StringBuffer  s_buf = new StringBuffer ( );
       int  length = s.length ( );
       boolean  in_quote = false;
       boolean  first_whitespace = true;
       for ( int  i = 0; i < length; i++ ) {
         char  c = s.charAt ( i );
         if ( c == '"' ) {
           in_quote = !in_quote;
           s_buf.append ( c );
           first_whitespace = true;
           continue;
         }
         if ( in_quote ) {
           s_buf.append ( c );
           continue;
         }
         if ( Character.isWhitespace ( c ) ) {
           if ( !first_whitespace ) continue;
           first_whitespace = false;
           s_buf.append ( ' ' );
         } else {
           first_whitespace = true;
           s_buf.append ( c );
         }
       }
       return s_buf.toString ( );
     }

     /*********************************************************************
     * Converts an array of bytes to a String of base 32 characters.
     * @see #toHexString
     *********************************************************************/
     public static String  toBase32String ( byte [ ]  digest )
     //////////////////////////////////////////////////////////////////////
     {
       StringBuffer  buf = new StringBuffer ( );
       for ( int  i = 0; i < digest.length; i++ )
       {
         String  b32 = Integer.toString ( digest [ i ], 32 );
         int  length = b32.length ( );
         if ( length < 2 ) b32 = '0' + b32;
         else b32 = b32.substring ( length - 2 );
         buf.append ( b32 );
       }
       return buf.toString ( );
     }

     /*********************************************************************
     * Converts an array of bytes to a String of hexadecimal characters.
     * @see #hexStringToByteArray
     *********************************************************************/
     public static String  toHexString ( byte [ ]  digest )
     //////////////////////////////////////////////////////////////////////
     {
       StringBuffer  buf = new StringBuffer ( );
       for ( int  i = 0; i < digest.length; i++ )
       {
         String  hex = Integer.toHexString ( digest [ i ] );
         int  length = hex.length ( );
         if ( length < 2 ) hex = '0' + hex;
         else hex = hex.substring ( length - 2 );
         buf.append ( hex );
       }
       return buf.toString ( );
     }

     /*********************************************************************
     * Lowercases everything outside of quotes.
     *********************************************************************/
     public static String  toLowerCaseOutsideQuotes ( String  s )
     //////////////////////////////////////////////////////////////////////
     {
       if ( s == null ) return null;
       StringBuffer  s_buf = new StringBuffer ( );
       int  length = s.length ( );
       boolean  inside_quotes = false;
       for ( int  i = 0; i < length; i++ ) {
         char  c = s.charAt ( i );
         if ( c == '"' )
         {
           inside_quotes = !inside_quotes;
           s_buf.append ( c );
           continue;
         }
         if ( !inside_quotes ) c = Character.toLowerCase ( c );
         s_buf.append ( c );
       }
       return s_buf.toString ( );
     }

     /*********************************************************************
     * Converts a String to an array of String by breaking it at the
     * newline ('\n') characters.  Strips out the newline characters in
     * the process.
     *********************************************************************/
     public static String [ ]  toStringArray (
       String  newline_separated_String )
     //////////////////////////////////////////////////////////////////////
     {
       if ( newline_separated_String == null ) return null;

       if ( newline_separated_String.indexOf ( '\n' ) < 0 ) {
         String [ ]  s_array = new String [ 1 ];
         s_array [ 0 ] = new String ( newline_separated_String );
         return s_array;
       }

       Vector  vector = new Vector ( );
       int  index     = 0;
       int  index_old = 0;

       while ( index >= 0 ) {
         index = newline_separated_String.indexOf ( '\n', index_old );
         if ( index < 0 ) {
           vector.addElement ( newline_separated_String.substring (
             index_old, newline_separated_String.length ( ) ) );
         } else if ( index == index_old ) {
           vector.addElement ( new String ( "" ) );
         } else {
           vector.addElement (
             newline_separated_String.substring ( index_old, index ) );
         }
         index_old = index + 1;
       }

       String [ ]  s_array = new String [ vector.size ( ) ];
       for ( int  i = 0; i < s_array.length; i++ ) {
         s_array [ i ] = new String ( ( String ) vector.elementAt ( i ) );
       }
       return s_array;
     }

     /*********************************************************************
     * Trims outer quotes.
     *********************************************************************/
     public static String  trimOuterQuotes ( String  s )
     //////////////////////////////////////////////////////////////////////
     {
       if ( s == null ) return null;
       if ( s.equals ( "" ) ) return "";
       if ( s.startsWith ( "\"" ) ) s = s.substring ( 1 );
       if ( s.endsWith ( "\"" ) ) s = s.substring ( 0, s.length ( ) - 1 );
       return new String ( s );
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
