     //////////////////////////////////////////////////////////////////////
     //
     // Copyright (C) 1998 Analytic Services, Inc. (ANSER)
     //
     // Permission is hereby granted, free of charge, to any person
     // obtaining a copy of this software and associated documentation
     // files (the "Software"), to deal in the Software without
     // restriction, including without limitation the rights to use, copy,
     // modify, merge, publish, distribute, sublicense, and/or sell copies
     // of the Software, and to permit persons to whom the Software is
     // furnished to do so, subject to the following conditions:
     //
     // The above copyright notice and this permission notice shall be
     // included in all copies or substantial portions of the Software.
     //
     // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
     // EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
     // MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
     // NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
     // HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
     // WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
     // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
     // DEALINGS IN THE SOFTWARE.
     //
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mit.servlet;

     import java.io.*;
     import java.net.*;
     import java.util.*;

     import javax.servlet.*;
     import javax.servlet.http.*;

     import com.orbs.open1.mit.app.ajax.*;
     import com.orbs.open1.mit.io.Serialize;

     /*********************************************************************
     * <P>
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft">David W. Croft</A>
     * @version
     *   1998-05-05
     *********************************************************************/

     public abstract class  AgentServlet extends AJAXServlet
       implements Runnable
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     public static final long  MS_PER_DAY = 1000 * 60 * 60 * 24;

     protected Thread     thread;
     protected Hashtable  user_Hashtable = new Hashtable ( );

// This will be replaced with something variable that depends on
//   when the next mission is ready to go.
     private static final long  SLEEP_PERIOD = MS_PER_DAY / 2;

     //////////////////////////////////////////////////////////////////////
     // Abstract methods
     //////////////////////////////////////////////////////////////////////

//   public abstract double  getVersion ( );
//   public abstract String  getServletInfo ( );
     public abstract String  getMissionFilename ( );

     public abstract void  processRequest (
       HttpServletRequest   req,
       HttpServletResponse  res,
       UserData             userData,
       Vector               mission_Vector )
       throws IOException;

     //////////////////////////////////////////////////////////////////////
     // Abstract method implementations
     //////////////////////////////////////////////////////////////////////

     /*********************************************************************
     * Returns true.
     *********************************************************************/
     protected boolean  requiresUserData ( ) { return true; }

     /*********************************************************************
     * Fetches or initializes the mission_Vector for the user and then
     * calls processRequest(req, res, userData, mission_Vector ).
     *********************************************************************/
     public void  processRequest (
       HttpServletRequest   req,
       HttpServletResponse  res,
       UserData             userData )
       throws IOException, ServletException
     //////////////////////////////////////////////////////////////////////
     {
       if ( userData == null ) return;
       String  name = userData.getName ( );

       Vector  mission_Vector = ( Vector ) user_Hashtable.get ( name );
       if ( mission_Vector == null )
       {
         mission_Vector = new Vector ( );
         user_Hashtable.put ( name, mission_Vector );
       }
       processRequest ( req, res, userData, mission_Vector );
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public void  init ( ServletConfig  config ) throws ServletException
     //////////////////////////////////////////////////////////////////////
     {
       super.init ( config );
       try
       {
         load ( );
         ( thread = new Thread ( this ) ).start ( );
       }
       catch ( Exception  ex ) { ex.printStackTrace ( ); }
     }

     public void  destroy ( )
     //////////////////////////////////////////////////////////////////////
     {
       save ( );
       if ( thread != null )
       {
         try { thread.stop ( ); } catch ( Throwable  t ) { }
         thread = null;
       }
       super.destroy ( );
     }

     protected synchronized boolean  load ( )
     //////////////////////////////////////////////////////////////////////
     {
       try
       {
         user_Hashtable
           = ( Hashtable ) Serialize.load ( getMissionFilename ( ) );
       }
       catch ( Exception  ex ) { ex.printStackTrace ( ); }
       if ( user_Hashtable == null )
       {
         user_Hashtable = new Hashtable ( );
         return false;
       }
       return true;
     }

     protected synchronized boolean  save ( )
     //////////////////////////////////////////////////////////////////////
     {
       return Serialize.save ( getMissionFilename ( ), user_Hashtable );
     }

     public final void  run ( )
     //////////////////////////////////////////////////////////////////////
     {
       try
       {
         while ( true )
         {
           Hashtable  clone = ( Hashtable ) user_Hashtable.clone ( );
           Enumeration  e = clone.elements ( );
           while ( e.hasMoreElements ( ) )
           {
             Vector  mission_Vector
               = ( Vector ) ( ( Vector ) e.nextElement ( ) ).clone ( );
             Enumeration  f = mission_Vector.elements ( );
             while ( f.hasMoreElements ( ) )
             {
               Mission  mission = ( Mission ) f.nextElement ( );
               mission.setAgentServlet ( this );
               mission.run ( );
             }
           }
           save ( );
           try { Thread.sleep ( SLEEP_PERIOD ); }
           catch ( InterruptedException  ie ) { }
         }
       }
       catch ( Exception  ex ) { ex.printStackTrace ( ); }
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
