     //////////////////////////////////////////////////////////////////////
     // The contents of this file are subject to the Mozilla Public License
     // Version 1.1 (the "License"); you may not use this file except in
     // compliance with the License.  You may obtain a copy of the License
     // at http://www.mozilla.org/MPL/
     //
     // Software distributed under the License is distributed on an "AS IS"
     // basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
     // See the License for the specific language governing rights and
     // limitations under the License.
     //
     // The Original Code is
     // "com.orbs.open1.mpl.app.chat.server.servlet.ChatServlet",
     // created 1999-11-28.
     //
     // The Initial Developer of the Original Code is David Wallace Croft
     // (croft@alumni.caltech.edu, http://www.alumni.caltech.edu/~croft/).
     // Portions created by David Wallace Croft are
     // Copyright (C) 2000 David Wallace Croft.  All Rights Reserved.
     //
     // Contributor(s):
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mpl.app.chat.server.servlet;

     import java.io.*;
     import java.util.*;

     import javax.servlet.*;
     import javax.servlet.http.*;

     import com.orbs.open1.mpl.j1d1.util.queue.*;

     import com.orbs.open1.mit.servlet.ServletLib;
     import com.orbs.open1.mit.util.param.*;

     import com.orbs.open1.mpl.app.chat.*;
     import com.orbs.open1.mpl.app.chat.server.*;

     /*********************************************************************
     *
     * <P>
     *
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft">David W. Croft</A>
     * @version
     *   2000-04-27
     *********************************************************************/

     public final class  ChatServlet extends HttpServlet
       implements ChatConstants
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     private static final long  TIME_TO_LIVE_WITHOUT_RENEWAL = 1000 * 60;

private static final long  TIMEOUT = 15000;

     private Map            idToOutgoingQueueMap;

     private ChatSequencer  chatSequencer;

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public void  init ( ServletConfig  servletConfig )
       throws ServletException
     //////////////////////////////////////////////////////////////////////
     {
       super.init ( servletConfig );

       try
       {
         idToOutgoingQueueMap = new HashMap ( );

         ChatEntityID  chatEntityID = new ChatEntityID ( -1 );

         ChatInvoker  chatInvoker = new ChatInvoker (
           chatEntityID,
           new ChatModel ( chatEntityID, idToOutgoingQueueMap,
           TIME_TO_LIVE_WITHOUT_RENEWAL ) );

         chatSequencer
           = new ChatSequencer ( chatInvoker, idToOutgoingQueueMap );
       }
       catch ( Exception  ex )
       {
         throw new UnavailableException ( this, ex.getMessage ( ) );
       }
     }

     public void  doGet (
       HttpServletRequest   req,
       HttpServletResponse  res )
       throws ServletException, IOException
     //////////////////////////////////////////////////////////////////////
     {
       doPost ( req, res );
     }

     public void  doPost (
       HttpServletRequest   httpServletRequest,
       HttpServletResponse  httpServletResponse )
       throws ServletException, IOException
     //////////////////////////////////////////////////////////////////////
     {
       try
       {
         Parameters  parameters
           = ServletLib.extractParameters ( httpServletRequest );

         if ( ( parameters == null )
           || ( parameters.getNames ( ).length < 1 ) )
         {
System.err.println ( "ChatServlet.serveApplet()" );

           serveApplet ( httpServletResponse );

           return;
         }
         else
         {
System.err.println ( "ChatServlet parameters length:  "
  + parameters.getNames ( ).length );
    
         }

         ChatMessage  requestChatMessage
           = ChatMessage.decode ( parameters );

         if ( requestChatMessage == null )
         {
System.err.println ( "ChatServlet:  null requestChatMessage" );

           httpServletResponse.setStatus (
             HttpServletResponse.SC_BAD_REQUEST );

           return;           
         }

         String  messageCode = requestChatMessage.getMessageCode ( );

         if ( messageCode == null )
         {
System.err.println ( "ChatServlet:  null messageCode" );

           httpServletResponse.setStatus (
             HttpServletResponse.SC_BAD_REQUEST );

           return;           
         }

System.err.println ( "ChatServlet receiving:  " + messageCode );

         if ( messageCode.equals ( ChatMessage.CODE_REQUEST_MESSAGE ) )
         {
           Queue  outgoingQueue = ( Queue ) idToOutgoingQueueMap.get (
             requestChatMessage.getChatEntityID ( ) );

           if ( outgoingQueue != null )
           {
             ChatMessage  responseChatMessage
               = ( ChatMessage ) outgoingQueue.pull ( TIMEOUT );

// If the Queue size drops to zip, should we delete it?

             if ( responseChatMessage == null )
             {
               httpServletResponse.setStatus (
                 HttpServletResponse.SC_NO_CONTENT );
             }
             else
             {
               httpServletResponse.setStatus (
                 HttpServletResponse.SC_OK );

System.err.println ( "ChatServlet transmission:  "
  + responseChatMessage.getMessageCode ( ) );

               OutputStream  outputStream
                 = httpServletResponse.getOutputStream ( );

               byte [ ]  bytes = ChatMessage.encode (
                 responseChatMessage ).getBytes ( );

               httpServletResponse.setContentLength ( bytes.length );

               outputStream.write ( bytes );

               outputStream.flush ( );
             }
           }
           else
           {
// FIX
// if there is no queue, this guy needs to reregister

             httpServletResponse.setStatus (
               HttpServletResponse.SC_NO_CONTENT );
           }
         }
         else
         {
           if ( !chatSequencer.insert ( requestChatMessage ) )
           {
             httpServletResponse.setStatus (
               HttpServletResponse.SC_SERVICE_UNAVAILABLE );
           }
           else
           {
             httpServletResponse.setStatus (
               HttpServletResponse.SC_ACCEPTED );
           }
         }
       }
       catch ( Exception  ex )
       {
         try
         {
           httpServletResponse.setStatus (
             HttpServletResponse.SC_INTERNAL_SERVER_ERROR );
         }
         catch ( Exception  ex1 )
         {
         }

         ex.printStackTrace ( );

         System.err.println ( ex );
       }
     }

     public void  destroy ( )
     //////////////////////////////////////////////////////////////////////
     {
// need to save messages in queues (in & out)

       chatSequencer.shutdown ( );

       super.destroy ( );
     }

     //////////////////////////////////////////////////////////////////////
     // Private methods
     //////////////////////////////////////////////////////////////////////

     public void  serveApplet ( HttpServletResponse  res )
       throws IOException
     //////////////////////////////////////////////////////////////////////
     {
       res.setContentType ( "text/html" );

       PrintStream  out = new PrintStream ( res.getOutputStream ( ) );

       try
       {
         out.println ( "<HTML><HEAD><TITLE>ORBS Chat 2000</TITLE></HEAD>\n"
           + "<BODY><CENTER>\n"
           + "<APPLET\n"
           + "  ARCHIVE=\"/java/ext/chatapplet.jar\"\n"
           + "  CODEBASE=\"/java/lib\"\n"
           + "  CODE=\"com.orbs.open1.mpl.app.chat.client.applet.ChatApplet.class\"\n"
           + "  WIDTH=468\n"
           + "  HEIGHT=255\n"
           + ">\n"
           + "A Java applet would be here.\n"
           + "</APPLET>\n"
           + "<HR>\n"
           + "<H1>ORBS Chat 2000</H1>\n"
           + "<HR>\n"
           + "</CENTER></BODY></HTML>" );
       }
       catch ( Throwable  t )
       {
         t.printStackTrace ( );
       }
       finally
       {
         try
         {
           out.flush ( );

//           out.close ( );
         }
         catch ( Exception  ex ) { }
       }
       
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
