     //////////////////////////////////////////////////////////////////////
     // The contents of this file are subject to the Mozilla Public License
     // Version 1.1 (the "License"); you may not use this file except in
     // compliance with the License.  You may obtain a copy of the License
     // at http://www.mozilla.org/MPL/
     //
     // Software distributed under the License is distributed on an "AS IS"
     // basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
     // See the License for the specific language governing rights and
     // limitations under the License.
     //
     // The Original Code is
     // "com.orbs.open1.mpl.app.chat.ChatMessagePuller",
     // created 1999-12-19.
     //
     // The Initial Developer of the Original Code is David Wallace Croft
     // (croft@alumni.caltech.edu, http://www.alumni.caltech.edu/~croft/).
     // Portions created by David Wallace Croft are
     // Copyright (C) 1999 David Wallace Croft.  All Rights Reserved.
     //
     // Contributor(s):
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mpl.j1d1.net.http.msg;

     import java.io.*;
     import java.net.*;

     import com.orbs.open1.mpl.j1d1.net.http.HttpConnection;
     import com.orbs.open1.mpl.j1d1.util.queue.*;

     import com.orbs.open1.mpl.io.Parser;
     import com.orbs.open1.mpl.util.loop.*;

//   import com.orbs.open1.mit.util.param.*;

     /*********************************************************************
     * Polls a server for messages and downloads them to a local queue.
     *
     * <P>
     *
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft">David W. Croft</A>
     * @version
     *   2000-04-27
     *********************************************************************/

     public final class  HttpMessagePuller implements Loopable
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     private static final long   POLLING_PERIOD_MAX  = 1000 * 60 * 60 * 24;
                                
     private static final float  POLLING_PERIOD_MULT = 1.1f;

     private static final long   POLLING_PERIOD_MIN  = 1000;

     private static final long   POLLING_PERIOD_INIT = 1000;

     //

     private final HttpConnection  httpConnection;

     private final Queue           incomingQueue;

     private final Looper          looper;

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public  HttpMessagePuller (
       String    host,
       int       port,
       String    path,
       String    userAgent,
       String    contentType,
       byte [ ]  requestBytes,
       Parser    contentParser,
       Queue     incomingQueue )
     //////////////////////////////////////////////////////////////////////
     {
       if ( host == null )
       {
         throw new IllegalArgumentException ( "null host" );
       }

       if ( path == null )
       {
         throw new IllegalArgumentException ( "null path" );
       }

       if ( userAgent == null )
       {
         throw new IllegalArgumentException ( "null userAgent" );
       }

       if ( contentType == null )
       {
         throw new IllegalArgumentException ( "null contentType" );
       }

       if ( requestBytes == null )
       {
         throw new IllegalArgumentException ( "null requestBytes" );
       }

       if ( contentParser == null )
       {
         throw new IllegalArgumentException ( "null contentParser" );
       }

       if ( incomingQueue == null )
       {
         throw new IllegalArgumentException ( "null incomingQueue" );
       }

       httpConnection = new HttpConnection ( host, port, path, userAgent,
         contentType, requestBytes, contentParser );
         
       this.incomingQueue = incomingQueue;

       looper = new Looper ( this, POLLING_PERIOD_INIT, null );

       looper.start ( );
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public void  setRequestBytes ( byte [ ]  requestBytes )
     //////////////////////////////////////////////////////////////////////
     {
       httpConnection.setRequestBytes ( requestBytes );
     }

     public synchronized long  setPollingPeriod ( long  pollingPeriod )
     //////////////////////////////////////////////////////////////////////
     {
       if ( pollingPeriod < POLLING_PERIOD_MIN )
       {
         pollingPeriod = POLLING_PERIOD_MIN;
       }

       if ( pollingPeriod > POLLING_PERIOD_MAX )
       {
         pollingPeriod = POLLING_PERIOD_MAX;
       }

       looper.setPeriod ( pollingPeriod );

       return pollingPeriod;       
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public boolean  start ( )
     //////////////////////////////////////////////////////////////////////
     {
       return looper.start ( );
     }

     public boolean  loop ( )
     //////////////////////////////////////////////////////////////////////
     {
System.out.println ( "HttpMessagePuller.loop()..." );

       try
       {
         httpConnection.post ( );

         int  responseCode = httpConnection.getResponseCode ( );

         if ( ( responseCode >= 200 )
           && ( responseCode <= 299 ) )
         {
           Object  messageObject = httpConnection.getParsedContent ( );

           if ( messageObject != null )
           {
             incomingQueue.append ( messageObject );
           }

           decreasePollingPeriod ( );
         }
         else
         {
           increasePollingPeriod ( );
         }

         return true;
       }
       catch ( Exception  ex )
       {
         ex.printStackTrace ( );

         increasePollingPeriod ( );

         return true;
       }
     }

     public void  stop ( )
     //////////////////////////////////////////////////////////////////////
     {
       looper.stop ( );
     }

     public void  abort ( )
     //////////////////////////////////////////////////////////////////////
     {
       httpConnection.abort ( );
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     private void  increasePollingPeriod ( )
     //////////////////////////////////////////////////////////////////////
     {
       long  pollingPeriod = looper.getPeriod ( );

       pollingPeriod = ( long ) ( pollingPeriod * POLLING_PERIOD_MULT );

       if ( pollingPeriod > POLLING_PERIOD_MAX )
       {
         pollingPeriod = POLLING_PERIOD_MAX;
       }

       looper.setPeriod ( pollingPeriod );
     }

     private void  decreasePollingPeriod ( )
     //////////////////////////////////////////////////////////////////////
     {
       long  pollingPeriod = looper.getPeriod ( );

       pollingPeriod = ( long ) ( pollingPeriod / POLLING_PERIOD_MULT );

       if ( pollingPeriod < POLLING_PERIOD_MIN )
       {
         pollingPeriod = POLLING_PERIOD_MIN;
       }

       looper.setPeriod ( pollingPeriod );
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
