     //////////////////////////////////////////////////////////////////////
     // The contents of this file are subject to the Mozilla Public License
     // Version 1.1 (the "License"); you may not use this file except in
     // compliance with the License.  You may obtain a copy of the License
     // at http://www.mozilla.org/MPL/
     //
     // Software distributed under the License is distributed on an "AS IS"
     // basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
     // See the License for the specific language governing rights and
     // limitations under the License.
     //
     // The Original Code is "com.orbs.open1.mpl.lang.StringLib",
     // created 1998-10-04.
     //
     // The Initial Developer of the Original Code is David Wallace Croft
     // (croft@alumni.caltech.edu, http://www.alumni.caltech.edu/~croft/).
     // Portions created by David Wallace Croft are
     // Copyright (C) 1999 David Wallace Croft.  All Rights Reserved.
     //
     // Contributor(s):
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mpl.lang;

     import java.util.Vector;

     /*********************************************************************
     *
     * A collection of static methods to manipulate java.lang.String.
     * 
     * @version
     *   1999-12-22
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft/">David W. Croft</A>
     *********************************************************************/

     public class  StringLib
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     public static final String [ ]  ZERO_LENGTH_STRING_ARRAY
       = new String [ ] { };

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public static void  main ( String [ ]  args )
     //////////////////////////////////////////////////////////////////////
     {
       System.out.println ( replace ( "0", "0", "12345" ) );
       System.out.println ( replace ( "1abc5", "abc", "234" ) );
       System.out.println ( replace ( "1a", "a", "2345" ) );
       System.out.println ( replace ( "1235", "23", "234" ) );
     }

     /*********************************************************************
     * @see padRight
     *********************************************************************/
     public static String  padLeft ( String  s, char  c, int  length )
     //////////////////////////////////////////////////////////////////////
     {
       if ( s == null ) return null;
       if ( s.length ( ) >= length ) return new String ( s );
       while ( s.length ( ) < length ) s = c + s;
       return s;
     }

     /*********************************************************************
     * @see padLeft
     *********************************************************************/
     public static String  padRight ( String  s, char  c, int  length )
     //////////////////////////////////////////////////////////////////////
     {
       if ( s == null ) return null;
       if ( s.length ( ) >= length ) return new String ( s );
       while ( s.length ( ) < length ) s = s + c;
       return s;
     }

     /*********************************************************************
     * Returns a String with the characters between
     * <I>fromIndex</I> and <I>toIndex</I> removed, inclusive.
     *********************************************************************/
     public static String  remove (
       String  s, int  fromIndex, int  toIndex )
     //////////////////////////////////////////////////////////////////////
     {
       return s.substring ( 0, fromIndex ) + s.substring ( toIndex + 1 );
     }

     /*********************************************************************
     * Replaces substrings in a String.
     * Replaces every instance of <I>oldSub</I> in <I>s</I> with
     *   <I>newSub</I>.
     *********************************************************************/
     public static String  replace (
       String  s, String  oldSub, String  newSub )
     //////////////////////////////////////////////////////////////////////
     // When Java 1.2 comes out, use StringBuffer.replace().
     //////////////////////////////////////////////////////////////////////
     {
       StringBuffer  sb = new StringBuffer ( );
       int  index;
       int  remnantIndex = 0;
       while ( ( index = s.indexOf ( oldSub, remnantIndex ) ) > -1 )
       {
         if ( index > remnantIndex )
         {
           sb.append ( s.substring ( remnantIndex, index ) );
         }
         sb.append ( newSub );
         remnantIndex = index + oldSub.length ( );
       }
       sb.append ( s.substring ( remnantIndex ) );
       return sb.toString ( );
     }

     /*********************************************************************
     * Converts a String to an array of String by breaking it at the
     * newline ('\n') characters.  Strips out the newline characters in
     * the process.
     *********************************************************************/
     public static String [ ]  toStringArray (
       String  newline_separated_String )
     //////////////////////////////////////////////////////////////////////
     {
       if ( newline_separated_String == null ) return null;

       if ( newline_separated_String.indexOf ( '\n' ) < 0 ) {
         String [ ]  s_array = new String [ 1 ];
         s_array [ 0 ] = new String ( newline_separated_String );
         return s_array;
       }

       Vector  vector = new Vector ( );
       int  index     = 0;
       int  index_old = 0;

       while ( index >= 0 ) {
         index = newline_separated_String.indexOf ( '\n', index_old );
         if ( index < 0 ) {
           vector.addElement ( newline_separated_String.substring (
             index_old, newline_separated_String.length ( ) ) );
         } else if ( index == index_old ) {
           vector.addElement ( new String ( "" ) );
         } else {
           vector.addElement (
             newline_separated_String.substring ( index_old, index ) );
         }
         index_old = index + 1;
       }

       String [ ]  s_array = new String [ vector.size ( ) ];
       for ( int  i = 0; i < s_array.length; i++ ) {
         s_array [ i ] = new String ( ( String ) vector.elementAt ( i ) );
       }
       return s_array;
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     private  StringLib ( ) { }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
