     //////////////////////////////////////////////////////////////////////
     // The contents of this file are subject to the Mozilla Public License
     // Version 1.1 (the "License"); you may not use this file except in
     // compliance with the License.  You may obtain a copy of the License
     // at http://www.mozilla.org/MPL/
     //
     // Software distributed under the License is distributed on an "AS IS"
     // basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
     // See the License for the specific language governing rights and
     // limitations under the License.
     //
     // The Original Code is "com.orbs.open1.mpl.math.RandomLib",
     // created 1999-01-31.
     //
     // The Initial Developer of the Original Code is David Wallace Croft
     // (croft@alumni.caltech.edu, http://www.alumni.caltech.edu/~croft/).
     // Portions created by David Wallace Croft are
     // Copyright (C) 1999 David Wallace Croft.  All Rights Reserved.
     //
     // Contributor(s):
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mpl.math;

     import java.util.Random;

     /*********************************************************************
     * Random number generation methods.
     *
     * @version
     *   1999-01-31
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft/">David W. Croft</A>
     *********************************************************************/

     public class  RandomLib
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     /** Seeded off of the current time. */
     private static Random  random = new Random ( );

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     /*********************************************************************
     * Test method; prints roll(arg0, arg1, arg2).
     *********************************************************************/
     public static void  main ( String [ ]  args )
     //////////////////////////////////////////////////////////////////////
     {
       long  multiplier = Long.parseLong ( args [ 0 ] );
       long  base       = Long.parseLong ( args [ 1 ] );
       long  offset     = Long.parseLong ( args [ 2 ] );
       System.out.println ( roll ( multiplier, base, offset ) );
     }

     /*********************************************************************
     * Returns a pseudorandom, uniformly distributed long value between 0
     * (inclusive) and the specified value (exclusive).
     *
     * @throws  IllegalArgumentException
     *   If n is negative.
     *********************************************************************/
     public static long  nextLong ( long  n )
     //////////////////////////////////////////////////////////////////////
     {
       if ( n < 0 )
       {
         throw new IllegalArgumentException ( "negative n:  " + n );
       }

       if ( n <= Integer.MAX_VALUE )
       {
         return ( long ) random.nextInt ( ( int ) n );
       }

       // This could be slow but I want to make sure it is uniform.
       long  roll = -1;
       while ( ( roll < 0 ) || ( roll >= n ) ) roll = random.nextLong ( );
       return roll;
     }

     /*********************************************************************
     * Rolls an n-sided die a specified number of times and adds an offset.
     * For example, to roll a 6-sided die 3 times and add 4 (3d6+4),
     * the multiplier would be 3, the base 6, and the offset 4.
     *
     * @param  multiplier
     *   The number of times to roll the die.  Must be non-negative.
     * @param  base
     *   The number of sides on the die, e.g., six.
     * @param  offset
     *   A final adjustment to add to the sum after all rolls have been
     *   made.
     * @return
     *   Returns the sum of the rolls plus the offset.
     *   The overflow condition is not handled.
     *********************************************************************/
     public static long  roll (
       long  multiplier,
       long  base,
       long  offset )
     //////////////////////////////////////////////////////////////////////
     {
       if ( multiplier < 0 )
       {
         throw new IllegalArgumentException (
           "negative multiplier:  " + multiplier );
       }

       long  sum = 0;

       for ( long  i = 0; i < multiplier; i++ )
       {
         sum += nextLong ( base ) + 1;
       }

       return sum + offset;
     }

     /*********************************************************************
     * Returns a double uniformly distributed between min (inclusive) and
     * max (exclusive).
     *********************************************************************/
     public static double  uniform ( double  min, double  max )
     //////////////////////////////////////////////////////////////////////
     {
       return min + ( max - min ) * random.nextDouble ( );
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
