     //////////////////////////////////////////////////////////////////////
     // The contents of this file are subject to the Mozilla Public License
     // Version 1.1 (the "License"); you may not use this file except in
     // compliance with the License.  You may obtain a copy of the License
     // at http://www.mozilla.org/MPL/
     //
     // Software distributed under the License is distributed on an "AS IS"
     // basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
     // See the License for the specific language governing rights and
     // limitations under the License.
     //
     // The Original Code is "com.orbs.open1.mpl.util.cache.Cache",
     // created 1999-04-08.
     //
     // The Initial Developer of the Original Code is David Wallace Croft
     // (croft@alumni.caltech.edu, http://www.alumni.caltech.edu/~croft/).
     // Portions created by David Wallace Croft are
     // Copyright (C) 1999 David Wallace Croft.  All Rights Reserved.
     //
     // Contributor(s):
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mpl.util.cache;

     import java.io.*;

     import com.orbs.open1.mpl.j1d1.util.id.ID;

     /*********************************************************************
     * Stores the content and then returns an ID which may be used to
     * read the content later, if still available.
     *
     * @see
     *   SoftCache
     * @see
     *   com.orbs.open.a.mpl.util.id.ID
     *
     * @version
     *   1999-04-24
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft/">David W. Croft</A>
     *********************************************************************/

     public interface  Cache
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     /*********************************************************************
     *
     * "Validates" the content by
     *
     * <OL>
     *
     * <LI> confirming that identical content already exists in the cache;
     *      or, if otherwise necessary,
     *
     * <LI> storing a new copy of the content in the cache.
     *
     * </OL>
     *
     * @param  id
     *
     *   The content identifier passed to isAvailable() to determine if
     *   the content is already valid.  The parameter may be any ID
     *   object that potentially matches via the equals() method.
     *
     * @param  contentAccessor
     *
     *   An object capable of making content accessible via an InputStream.
     *   For example, a ContentAccessor might retrieve content from a
     *   website via a URL, a database or file storage, a remote object
     *   such as another cache, or even dynamically generate the content
     *   upon demand.  As yet another possibility, a ContentAccessor object
     *   may potentially attempt to access the content from several
     *   different sources sequentially until it is successful.
     *
     * @return
     *
     *   Returns an ID object for the validated content which may be
     *   used later for retrieval.
     *
     *   <P>
     *
     *   If valid content was already available in the cache, the returned
     *   ID object will be the id parameter.
     *
     *   <P>
     *
     *   If valid content was not already available and the content could
     *   not be accessed and stored via the contentAccessor, the returned
     *   value will be null.
     *
     *   <P>
     *
     *   If valid content was not already available and the content could
     *   be accessed and stored via the contentAccessor, the returned
     *   value will be a new ID object with values that may or may
     *   not equal that of the id object parameter, depending on
     *   the actual content available via the contentAccessor.
     *
     *********************************************************************/
     public ID  validate ( ID  id, ContentAccessor  contentAccessor )
       throws IOException;

     /*********************************************************************
     * Stores the contents and returns an ID to be used for retrieval.
     *
     * <P>
     *
     * Reads the stream until completion and closes it before return.
     *
     * @return
     *   Returns an ID to be used for later retrieval.
     *   Returns null if the storage was unsuccessful.
     *********************************************************************/
     public ID  store ( InputStream  in ) throws IOException;

     /*********************************************************************
     * Retrieves the content associated with this ID.
     *
     * @param  id
     *   Returns the content associated with this id or its equivalent.
     * @return
     *   May return null.
     *********************************************************************/
     public InputStream  retrieve ( ID  id ) throws IOException;

     /*********************************************************************
     * Returns false if the content is no longer available.
     *
     * @param  id
     *   An ID object to be used to retrieve the content.
     *********************************************************************/
     public boolean  isAvailable ( ID  id );

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
