     //////////////////////////////////////////////////////////////////////
     // The contents of this file are subject to the Mozilla Public License
     // Version 1.1 (the "License"); you may not use this file except in
     // compliance with the License.  You may obtain a copy of the License
     // at http://www.mozilla.org/MPL/
     //
     // Software distributed under the License is distributed on an "AS IS"
     // basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
     // See the License for the specific language governing rights and
     // limitations under the License.
     //
     // The Original Code is "com.orbs.open1.mpl.util.cache.CacheLib",
     // created 1999-04-10.
     //
     // The Initial Developer of the Original Code is David Wallace Croft
     // (croft@alumni.caltech.edu, http://www.alumni.caltech.edu/~croft/).
     // Portions created by David Wallace Croft are
     // Copyright (C) 1999 David Wallace Croft.  All Rights Reserved.
     //
     // Contributor(s):
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mpl.util.cache;

     import java.io.*;

     import com.orbs.open1.mpl.j1d1.util.id.ID;
     import com.orbs.open1.mpl.j1d1.util.id.IntID;

     /*********************************************************************
     * Static methods to support Cache implementations.
     *
     * @see
     *   Cache
     * @see
     *   com.orbs.open.a.mpl.util.id.ID
     *
     * @version
     *   1999-04-20
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft/">David W. Croft</A>
     *********************************************************************/

     public class  CacheLib
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     private  CacheLib ( ) { }

     //////////////////////////////////////////////////////////////////////
     // Test methods
     //////////////////////////////////////////////////////////////////////

     /*********************************************************************
     * Tests Cache implementations.
     *
     * <PRE>
     *
     * if ( args.length < 1 )
     * {
     *   System.out.println ( test ( ) );
     * }
     * else
     * {
     *   System.out.println ( test ( args ) );
     * }
     *
     * </PRE>
     *
     * @param  args
     *   Names of Cache classes.
     *********************************************************************/
/*
     public static void  main ( String [ ]  args )
     //////////////////////////////////////////////////////////////////////
     {
       if ( args.length < 1 )
       {
         System.out.println ( test ( ) );
       }
       else
       {
         System.out.println ( test ( args ) );
       }
     }
*/

     /*********************************************************************
     * Tests the Cache implementations provided in this package.
     *********************************************************************/
/*
     public static boolean  test ( )
     //////////////////////////////////////////////////////////////////////
     {
       try
       {
         return test ( new SoftCache ( ) )
           &&   test ( new WeakCache ( ) );
       }
       catch ( Throwable  t )
       {
         t.printStackTrace ( );
         return false;
       }
     }
*/

     /*********************************************************************
     * Tests Cache implementations.
     *
     * @param  cacheClassNames
     *   Names of Cache classes.
     *********************************************************************/
/*
     public static boolean  test ( String [ ]  cacheClassNames )
     //////////////////////////////////////////////////////////////////////
     {
       try
       {
         boolean  result = true;

         for ( int  i = 0; i < cacheClassNames.length; i++ )
         {
           Class  c = Class.forName ( cacheClassNames [ i ] );
           Cache  cache = ( Cache ) c.newInstance ( );
           result = test ( cache ) && result;
           if ( !result ) break;
         }

         return result;
       }
       catch ( Throwable  t )
       {
         t.printStackTrace ( );
         return false;
       }
     }
*/

     /*********************************************************************
     * Tests a Cache implementation.
     *
     * @param  cache
     *   An instance of a Cache to be tested.
     *********************************************************************/
/*
     public static boolean  test ( Cache  cache )
     //////////////////////////////////////////////////////////////////////
     {
       try
       {
         Runtime  runtime = Runtime.getRuntime ( );

         ID  id0 = null;

         for ( int  index = 0; index < 1000; index++ )
         {
           // System.gc ( );

           if ( index % 10 == 0 )
           {
             System.out.println (
               index + "-:  "
               + runtime.freeMemory ( )
               + " / "
               + runtime.totalMemory ( ) );
           }

           String  s = Integer.toString ( index );
           ID  id = new IntegerID ( index );
           id = ( ID ) storeString ( cache, s, id ).clone ( );

           if ( index == 0 )
           {
             id0 = id;
           }

           if ( index % 10 == 0 )
           {
             System.out.println (
               index + "+:  "
               + runtime.freeMemory ( )
               + " / "
               + runtime.totalMemory ( )
               + " "
               + retrieveString ( cache, id0 ) );
           }

           if ( !s.equals ( retrieveString ( cache, id ) ) )
           {
             return false;
           }
         }

         return true;
       }
       catch ( Throwable  t )
       {
         t.printStackTrace ( );
         return false;
       }
     }
*/

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public static ID  storeString ( Cache  cache, String  s )
       throws IOException
     //////////////////////////////////////////////////////////////////////
     {
       return cache.store ( toInputStream ( s ) );
     }

     public static String  retrieveString ( Cache  cache, ID  id )
       throws IOException
     //////////////////////////////////////////////////////////////////////
     {
       return toString ( cache.retrieve ( id ) );
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public static byte [ ]  toByteArray ( InputStream  in )
       throws IOException
     //////////////////////////////////////////////////////////////////////
     {
       ByteArrayOutputStream  out = null;

       try
       {
         out = new ByteArrayOutputStream ( );

         int  i;
         while ( ( i = in.read ( ) ) > -1 ) out.write ( i );

         return out.toByteArray ( );
       }
       finally
       {
         try { in.close  ( ); } catch ( Exception  ex ) { }
         try { out.close ( ); } catch ( Exception  ex ) { }
       }
     }

     public static InputStream  toInputStream ( String  s )
     //////////////////////////////////////////////////////////////////////
     {
       return toInputStream ( s.getBytes ( ) );
     }

     public static InputStream  toInputStream ( byte [ ]  byteArray )
     //////////////////////////////////////////////////////////////////////
     {
       return new ByteArrayInputStream ( byteArray );
     }

     public static String  toString ( InputStream  in ) throws IOException
     //////////////////////////////////////////////////////////////////////
     {
       if ( in == null ) return null;

       ByteArrayOutputStream  out = new ByteArrayOutputStream ( );
       int  i;
       while ( ( i = in.read ( ) ) > -1 ) out.write ( i );

       return new String ( out.toByteArray ( ) );
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
