     //////////////////////////////////////////////////////////////////////
     // The contents of this file are subject to the Mozilla Public License
     // Version 1.1 (the "License"); you may not use this file except in
     // compliance with the License.  You may obtain a copy of the License
     // at http://www.mozilla.org/MPL/
     //
     // Software distributed under the License is distributed on an "AS IS"
     // basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
     // See the License for the specific language governing rights and
     // limitations under the License.
     //
     // The Original Code is "com.orbs.open1.mpl.util.state.StateLib",
     // created 1999-02-06.
     //
     // The Initial Developer of the Original Code is David Wallace Croft
     // (croft@alumni.caltech.edu, http://www.alumni.caltech.edu/~croft/).
     // Portions created by David Wallace Croft are
     // Copyright (C) 1999 David Wallace Croft.  All Rights Reserved.
     //
     // Contributor(s):
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mpl.util.state;

     /*********************************************************************
     *
     * Static methods to support concrete implementations of the State
     * interface.
     *
     * <P>
     *
     * Reference:<BR>
     * Mark Roulo,
     * <A HREF="http://www.javaworld.com/javaworld/jw-01-1999/jw-01-object.html">
     * "How to avoid traps and correctly override methods
     * from java.lang.Object"</A>, January 1999, JavaWorld.
     *
     * <P>
     *
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft/">David W. Croft</A>
     * @version
     *   1999-02-06
     *********************************************************************/

     public class  StateLib
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     private  StateLib ( ) { }

     /*********************************************************************
     * Returns true if the classes and State keys are equal.
     *
     * State implementation usage:
     * <PRE>
     * public boolean  equals ( Object  other )
     * {
     *   return StateLib.equals ( this, other );
     * }
     * </PRE>
     *********************************************************************/
     public static boolean  equals ( State  state, Object  other )
     //////////////////////////////////////////////////////////////////////
     {
       if ( state == null )
       {
         throw new IllegalArgumentException ( "null state" );
       }

       if ( ( other == null )
         || !other.getClass ( ).equals ( state.getClass ( ) ) )
       {
         return false;
       }

       return state.getKey ( ).equals ( ( ( State ) other ).getKey ( ) );
     }

     /*********************************************************************
     * Returns the hash code of the State key.
     *
     * <PRE>
     *   return state.getKey ( ).hashCode ( );
     * </PRE>
     *
     * State implementation usage:
     * <PRE>
     * public int  hashCode ( )
     * {
     *   return StateLib.hashCode ( this );
     * }
     * </PRE>
     *********************************************************************/
     public static int  hashCode ( State  state )
     //////////////////////////////////////////////////////////////////////
     {
       return state.getKey ( ).hashCode ( );
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
